Attribute VB_Name = "Common"
' --- GPL ---
'
' Copyright (C) 2000-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
' Module:      Common - Common Module
'
'-----------------------------------------------------------------------
'
' responsible:  WolfgangA
'
' special area: SQLStudio
'
' description:  module for global varibles, functions ...
'
'
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Option Explicit

'--------------------------------------------------------
' Module Name
'--------------------------------------------------------
Const MODULE_NAME = "SQLStudio.Common"

'=global public variables================================================

'global object for registry functions
Public m_oRegFunc                  As New SQLStudioUtil.Registry

'global object for saving/reading LRU connections in the registry
Public m_oLRUConnections           As New LRUConnections

'global object for the suppose list used in DSQL Dialog
Public m_oAutoSuppose              As New SQLStudioCCon.ASObject

'global object for
Public m_oStatementList            As SQLStudioUtil.History

'global object for communication between the application and some controls for sql studio
Public m_oGlobalInfo                As SQLStudioMan.globalInfo

'global object for a database connection
Public m_oAdminCon                  As SQLStudioDBLayer.connection

'global variable for the driver name, to be registered as odbc driver
Public m_sDriverName               As String

'global variable for maximum numbers of forems, which can be opened
Public m_csMaxForm                 As Long

'with a commandline option, the initial connect ,ode can be set to aracle
'remember with this variable
Public m_emInitConMode          As sqlMode

'counter for newly openend windows
Public m_lSQLDialogCounter        As Long
Public m_lFormDialogCounter       As Long
Public m_lVIQCounter              As Long
Public m_lNewTableCounter         As Long
Public m_lAlterTableCounter       As Long
Public m_lNewSeqCounter           As Long
Public m_lNewIndexCounter         As Long
Public m_lNewViewCounter          As Long
Public m_lNewSynonymCounter       As Long

'info what folders have been open in the save dialog
Public m_sLastOpenFolders         As String
Public m_sLastSelectedFolder      As String

Public m_oHelp                    As New HtmlHelp

'=========================================================================

'=global private variables================================================

'global object for common functions
Public m_oComFunc                 As New SQLStudioUtil.Functions

'global object for command line interface
Private m_oCML                    As New cmdLine

'===========================================================================


'=constants=================================================================
Const CML_OPT_HELP = "?"
Const CML_OPT_NODE = "n"
Const CML_OPT_DB = "d"
Const CML_OPT_USER = "u"
Const CML_OPT_NI = "O1"
Const CML_OPT_CM = "c"
Const CML_OPT_VER = "version"
Const CML_OPT_TRACE = "t"

Const cnstORACLE = "oracle"


Public Const constMinWidth = 1024
Public Const constMinHeight = 1024

Public Const LOCALFOLDERIND = "LocalFolderInd"

Public Const DOCKINGSUBKEY = "Docking"
Public Const VISIBLESUBKEY = "Visible"

'registry key names
Public Const m_cnstKEYNAMEDRIVER = "DRIVER"
Public Const m_cnstKEYNAMEUID = "UID"
Public Const m_cnstKEYNAMEPWD = "PWD"
Public Const m_cnstKEYDATABASE = "SERVERDB"
Public Const m_cnstKEYSERVERNODE = "SERVERNODE"

'========================================================================================

'=========================================================================================
'    function:      internalError
'    description:   shows message box with error
'    arguments:     [in] CurrentError, vb error object
'                   [in] Modulname, name of source modul
'                   [in] FunctionName, name of used function
'    return value:  /
'
'=========================================================================================
Public Function internalError(ByVal CurrentError As ErrObject, ModulName As String, FunctionName As String)
    MsgBox "Module:" + Chr$(9) + ModulName + Chr$(13) + Chr$(10) + "Function:" + Chr$(9) + FunctionName + Chr$(13) + Chr$(10) + "Error No:" + Chr$(9) + Trim$(Str$(CurrentError.Number)) + Chr$(13) + Chr$(10) + "Error:" + Chr$(9) + CurrentError.description, vbExclamation, "Unexpected Error:"
End Function

'=========================================================================================
'    function:      Main
'    description:   called when starting application
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Sub Main()
    On Error GoTo ErrMain
    
    Const FUNCTION_NAME = "Main"
    
    '=varibles===================================================================
    'buffer for command line string
    Dim sCommandLine As String
    
    'instance of about form
    Dim oAbout       As New SQLStudioCCon.About
    '============================================================================
    
    'set the driver name to fixed value
    m_sDriverName = ODBC_DRVR_NAME
    
    copyRegValuesFromVersion73010
        
    'number of max forms allowed
    m_csMaxForm = 33
        
    'set inital connect mode
    m_emInitConMode = Internal
    
    oAbout.Files.add "SQLSTO.exe", App.Path
    'show version from main programm
    If (Len(oAbout.Files.item(1).versionString) > 3) Then
    
        oAbout.version = Left(oAbout.Files.item(1).versionString, 3)
    Else
        oAbout.version = oAbout.Files.item(1).versionString
    End If
    
    oAbout.showSplashForm 1500, True, True
            
    'show main window
    frmMain.Show
    
    'deactive this menus
    frmMain.mnuViewStatusWindow.Checked = False
    frmMain.mnuViewCatalogManager.Checked = False
    frmMain.tbrMain.Buttons.item("Execute").Enabled = False
    
    If App.StartMode = vbSModeStandalone Then
    
        'create global info object
        createGlobalInfo
        'get command line
        sCommandLine = Command
        'init command line object
        cmdLineOptsInit
        If Len(sCommandLine) > 0 Then
        
            'analyze command line
            If checkCommandLine(sCommandLine) = True Then
            
                'connect to database
                If (connect() = False) Then
                    'End
                End If
            End If
        End If
    End If

                       
    Exit Sub

ErrMain:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      checkCommandLine
'    description:   analyzes a string for command line options like -d, -n ...
'    arguments:     [in] sCommandline, string to analyze
'
'    return value:  true if command line triggers a db connect
'
'=========================================================================================
Function checkCommandLine(sCommandLine As String) As Boolean
    On Error GoTo errCheckCommandLine
    
    Const FUNCTION_NAME = "checkCommandLine"
    
    '=varibles===================================================================
    Dim lPos    As Long
    Dim sBuffer As String
    '============================================================================
    
    'rc
    checkCommandLine = False
       
    
    m_oCML.Text = sCommandLine
    
    If m_oCML(CML_OPT_HELP).inCmdLine Then
        Set frmCmdLineOpt.cmdLine = m_oCML
        frmCmdLineOpt.Show vbModal
        Exit Function
    End If
    
    If m_oCML(CML_OPT_VER).inCmdLine Then
        showAboutForm
        Exit Function
    End If
    
    'if sql should run in oracle mode then put value to userprofile
    If m_oCML(CML_OPT_CM).Value = cnstORACLE Then
    
        m_emInitConMode = Oracle
    Else
        m_emInitConMode = Internal
    End If
    
    'set trace file
    m_oGlobalInfo.conMan.traceFile = m_oCML(CML_OPT_TRACE).Value
            
    If m_oCML.exists(CML_OPT_NI) Then
    
        m_oGlobalInfo.conMan.dbServer = m_oCML(CML_OPT_NI).Value
        m_oGlobalInfo.conMan.database = ""
        m_oGlobalInfo.conMan.UID = ""
    Else
        If m_oCML(CML_OPT_NODE).inCmdLine Or _
           m_oCML(CML_OPT_DB).inCmdLine Or _
           m_oCML(CML_OPT_USER).inCmdLine _
        Then
            m_oGlobalInfo.conMan.dbServer = m_oCML(CML_OPT_NODE).Value
            m_oGlobalInfo.conMan.database = m_oCML(CML_OPT_DB).Value
            
            sBuffer = m_oCML(CML_OPT_USER).Value
            lPos = InStr(sBuffer, Chr(34) + "," + Chr(34))
            'quoated uid,pwd
            If lPos > 0 Then
            
                m_oGlobalInfo.conMan.UID = Left(sBuffer, lPos - 1)
                m_oGlobalInfo.conMan.PWD = Right(sBuffer, Len(sBuffer) - lPos - 2)
            Else
                lPos = InStr(sBuffer, ",")
                'quoated uid,pwd
                If lPos > 0 Then
                
                    m_oGlobalInfo.conMan.UID = UCase(Left(sBuffer, lPos - 1))
                    m_oGlobalInfo.conMan.PWD = UCase(Right(sBuffer, Len(sBuffer) - lPos))
                Else
                    m_oGlobalInfo.conMan.UID = sBuffer
                End If
            End If
        End If
    End If
    
    'rc
    checkCommandLine = True
            
    Exit Function

errCheckCommandLine:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      connect
'    description:   connect to database
'    arguments:     /
'
'    return value:  true if successfully
'
'=========================================================================================
Function connect() As Boolean
    On Error GoTo errConnect
    
    Const FUNCTION_NAME = "connect"
       
    '=varibles===================================================================
    Dim bConnectSuccess    As Boolean
    Dim sDriverName        As String
    Dim sLocalRoot         As String
    Dim lRun               As Long
    Dim lValue             As Long
    Dim oConnection        As SQLStudioDBLayer.connection
    Dim emSQLMode          As sqlMode
    '============================================================================
    'rc
    connect = False
    'init
    bConnectSuccess = False
    Set m_oAdminCon = Nothing
    
    'check
    If m_oGlobalInfo Is Nothing Then
    
        MsgBox LoadResString(350), vbCritical
        End
    End If
    
    'init counters
    m_lSQLDialogCounter = 0
    m_lFormDialogCounter = 0
    m_lVIQCounter = 0
    m_lNewTableCounter = 0
    m_lAlterTableCounter = 0
    m_lNewSeqCounter = 0
    m_lNewIndexCounter = 0
    m_lNewViewCounter = 0

    
    'create history list for statements in dsql dialog
    Set m_oStatementList = m_oComFunc.createHistory(100)
    m_oStatementList.checkDoubleEntry = deStringGeneral
            
    'if no ODBC Driver is installed exit function
    If Len(m_sDriverName) < 1 Then
    
        MsgBox LoadResString(360)
        Exit Function
    End If
    
    '====================================================================
    'set pooling mode for conection manager
    '
    '     WA_DBC_EXCL: The returned connection is
    '         currently unused And won 't be returned
    '         for another Connect until Disconnect is called.
    '     WA_DBC_EXCL_AUTOCOMMIT_OFF: Only autocommit
    '         off connections are exclusiv. Autocommit on
    '         connections are shared.
    '     WA_DBC_SHARED: The connection may be returned
    '         by several Connects. Only if
    '         #Connects-#Disconntecs is 0 the connection can be
    '         used exclusive.
    '====================================================================
    
    'check for NO timeout value
    
    ' !!! set this value before poolingMode because it is considered by poolingMode
    If frmMain.chkNoTimeOut.Value = vbChecked Then _
       m_oGlobalInfo.conMan.timeout = 0
       
    m_oGlobalInfo.conMan.poolingMode = WA_DBC_EXCL_AUTOCOMMIT_OFF
    
    'set driver name
    m_oGlobalInfo.conMan.driver = m_sDriverName
    
    'check the timeout value for ni connection
    'the value has to be determined before connect
    'if the connection params are like this
    'we can asume its a ni call
    If m_oGlobalInfo.conMan.dbServer <> "" And _
        m_oGlobalInfo.conMan.database = "" And _
        m_oGlobalInfo.conMan.UID = "" And _
        m_oGlobalInfo.conMan.PWD = "" Then
        
        frmMain.chkNoTimeOut.Value = vbChecked
        m_oGlobalInfo.conMan.timeout = 0
    End If
    
    'ADIS ErrorId Begin: 1000796
    If m_oGlobalInfo.conMan.HWnd <> frmMain.HWnd Then
        m_oGlobalInfo.conMan.HWnd = frmMain.HWnd
    End If
    'ADIS ErrorId End: 1000796
    
    'turn hour glass on
    m_oGlobalInfo.Hourglass.State = qpHourglassOn
    
   '"Connecting ..."
    setState LoadResString(101), False
            
    'get user profile from the registry before connecting
    If (m_oGlobalInfo.userProfile.checkProfileKey() = True) Then
        m_oGlobalInfo.userProfile.loadUserProfileFromReg True
    End If
    
    'overwrite value from userprofile with value set by command line
    If m_emInitConMode = Oracle Then
        m_oGlobalInfo.conMan.driverSQLMode = m_emInitConMode
        m_oGlobalInfo.userProfile.appManagment.sqlMode = m_emInitConMode
    End If
            
    'set some properties for the connection manager
    Set m_oGlobalInfo.conMan.protWindow = frmMain.ctlStatusWindow.protWindow
    m_oGlobalInfo.conMan.statusVerbose = m_oGlobalInfo.userProfile.statusVerbose
    
    'get the first connection
    bConnectSuccess = m_oGlobalInfo.conMan.getConnection( _
                           m_oGlobalInfo.userProfile.appManagment.isoLevel, _
                           m_oGlobalInfo.userProfile.appManagment.sqlMode, _
                           AutoCommitOn, _
                           oConnection)
                           
    If Not bConnectSuccess Then
    
        'end application if ni mode is active
        If m_oGlobalInfo.conMan.isNIConnection Then End
    
        '"Connection failed ..."
        setState LoadResString(100), True
        
        'turn hour glass off
        m_oGlobalInfo.Hourglass.State = qpHourglassOff
        
        'if there is no connection exit function
        If oConnection Is Nothing Then Exit Function
        
        'if there is a error
        If Len(oConnection.Error.Text) > 0 Then
        
            'show error
            oConnection.Error.showError
            'end application if ni mode is active
            If Not oConnection Is Nothing Then
                If oConnection.isNIConnection Then End
            End If
            
            'react on this sql state
            If oConnection.Error.sqlState = "[ODBC] [IM002] [ODBC Driver Manager] Data source name not found and no default driver specified" Then
            
                m_oGlobalInfo.conMan.UID = ""
                m_oGlobalInfo.conMan.PWD = ""
                m_oGlobalInfo.conMan.database = ""
                m_oGlobalInfo.conMan.dbServer = ""
                                
                bConnectSuccess = m_oGlobalInfo.conMan.getConnection( _
                                      m_oGlobalInfo.conMan.driverIsoLevel, _
                                      m_oGlobalInfo.conMan.driverSQLMode, _
                                      AutoCommitOn, _
                                      oConnection)
                If Not bConnectSuccess Then
                
                     If Len(oConnection.Error.Text) > 0 Then oConnection.Error.showError
                        Exit Function
                End If
            End If
        Else
            If Len(oConnection.Error.sqlState) > 0 Then
                MsgBox oConnection.Error.sqlState
             Else
                showError142 oConnection
            End If
        End If
        
        'end application if ni mode is active
        If Not oConnection Is Nothing Then
            If oConnection.isNIConnection Then End
        End If
            
        Exit Function
    Else
        If Len(oConnection.userGroup) < 1 Then
        'there are no systemtables, open auto off connection
        
            MsgBox LoadResString(116), vbCritical
            '===========================================================
            'first disconnect autocommit on
            'release connection, very important for pooling mode <> WA_NONE
            m_oGlobalInfo.conMan.disconnect
            
            ' !!! set this value before poolingMode because it is considered by poolingMode
            If frmMain.chkNoTimeOut.Value = vbChecked Then _
                m_oGlobalInfo.conMan.timeout = 0
       
            m_oGlobalInfo.conMan.poolingMode = WA_DBC_EXCL_AUTOCOMMIT_OFF
            '===========================================================
            
            m_oGlobalInfo.conMan.PWD = ""
            'reinit pointer
            Set oConnection = Nothing
            'get auto off connection
            bConnectSuccess = m_oGlobalInfo.conMan.getConnection( _
                                     m_oGlobalInfo.conMan.driverIsoLevel, _
                                     m_oGlobalInfo.conMan.driverSQLMode, _
                                     AutoCommitOff, _
                                     oConnection)
            If Not bConnectSuccess Then
            
                m_oGlobalInfo.Hourglass.State = qpHourglassOff
                
                If Len(oConnection.Error.Text) > 0 Then oConnection.Error.showError
                Exit Function
            Else
               'open one DirectSQLWindow
               m_oGlobalInfo.systemtablesLoaded = False
               
               'set userprofile properties
               m_oGlobalInfo.userProfile.confirmation = False
               m_oGlobalInfo.userProfile.storePassword = False
               
               MsgBox LoadResString(143) + _
                        " (" + _
                        LoadResString(146) + oConnection.sqlModeAsString + _
                        ", " + _
                        LoadResString(145) + oConnection.isoLevelAsString + _
                        ", " + _
                        LoadResString(144) + ")", _
                        vbOKOnly
               
               showDirectSQLAutoCommitOff oConnection
            End If
        Else
            m_oGlobalInfo.systemtablesLoaded = True
        End If
    End If
       
    m_oGlobalInfo.Hourglass.State = qpHourglassOff
            
    '==================================================================
    'now there is a valid connection, remember it, use it as an admin connection
    '==================================================================
    Set m_oAdminCon = oConnection
    
    'always enable view menu
     frmMain.mnuView.Enabled = True
    
                
    frmMain.mnuWindow.Enabled = True
    
    'enable some menus if necessary systemtables are loaded
    frmMain.mnuViewDirectSQL.Enabled = m_oGlobalInfo.systemtablesLoaded
    frmMain.mnuViewDirectSQLAC.Enabled = m_oGlobalInfo.systemtablesLoaded
    frmMain.mnuViewNewQueryByExample.Enabled = m_oGlobalInfo.systemtablesLoaded
    frmMain.mnuViewNewVisualQuery.Enabled = m_oGlobalInfo.systemtablesLoaded
    
    'enable some buttons from the toolbar
    frmMain.tbrMain.Buttons.item("SQLDialogACOff").Enabled = m_oGlobalInfo.conMan.isNotExclusive And m_oGlobalInfo.systemtablesLoaded
    frmMain.tbrMain.Buttons.item("VisualQuery").Enabled = m_oGlobalInfo.systemtablesLoaded
    frmMain.tbrMain.Buttons.item("FormDialog").Enabled = m_oGlobalInfo.systemtablesLoaded
    
    frmMain.tbrMain.Buttons.item("CreateTable").Enabled = m_oGlobalInfo.systemtablesLoaded And m_oGlobalInfo.conMan.isResourceUser And m_oGlobalInfo.conMan.isNotExclusive
    frmMain.tbrMain.Buttons.item("CreateView").Enabled = m_oGlobalInfo.systemtablesLoaded And m_oGlobalInfo.conMan.isResourceUser And m_oGlobalInfo.conMan.isNotExclusive
    frmMain.tbrMain.Buttons.item("CreateSynonym").Enabled = m_oGlobalInfo.systemtablesLoaded And m_oGlobalInfo.conMan.isResourceUser And m_oGlobalInfo.conMan.isNotExclusive
    frmMain.tbrMain.Buttons.item("CreateIndex").Enabled = m_oGlobalInfo.systemtablesLoaded And m_oGlobalInfo.conMan.isResourceUser And m_oGlobalInfo.conMan.isNotExclusive
    frmMain.tbrMain.Buttons.item("CreateSequence").Enabled = m_oGlobalInfo.systemtablesLoaded And m_oGlobalInfo.conMan.isResourceUser And m_oGlobalInfo.conMan.isNotExclusive
      
    'enable some buttons from the toolbar
    frmMain.tbrMain.Buttons.item("SQLDialog").Enabled = m_oGlobalInfo.systemtablesLoaded
    frmMain.tbrMain.Buttons.item("Connect").Image = "Disconnect"
    frmMain.tbrMain.Buttons.item("Connect").ToolTipText = "Disconnect"
    
    frmMain.mnuViewCatalogManager.Enabled = True
    frmMain.tbrMain.Buttons(CATALOGMANAGER).Enabled = True
    
    frmMain.mnuViewStatusWindow.Enabled = True
    frmMain.tbrMain.Buttons(STATUSWINDOW).Enabled = True
       
    'overwrite value from userprofile with value set by command line
    If m_emInitConMode = Oracle Then
        m_oGlobalInfo.userProfile.appManagment.sqlMode = Oracle
    End If
           
    'check local root
    sLocalRoot = m_oGlobalInfo.userProfile.localRoot
    Call checkLocalRoot(sLocalRoot)
    m_oGlobalInfo.userProfile.localRoot = sLocalRoot
       
    'check for NI
    'PTS 1118499
    'If m_oAdminCon.isNIConnection Then frmMain.chkWanMode.Value = vbChecked
                
    Set frmMain.ctlStatusWindow.globalInfo = m_oGlobalInfo
    Set frmMain.ctlCM.globalInfo = m_oGlobalInfo
    'set a connection object
    Set frmMain.ctlCM.connection = m_oAdminCon
    'global info is set and userprofile was loaded
    frmMain.ctlCM.loadTableFilterList
    
               
    If m_oGlobalInfo.systemtablesLoaded Then
        
        'load supposelist
        m_oAutoSuppose.supposeTypeSQL = True
        m_oAutoSuppose.supposeTypeInfo = True
        m_oAutoSuppose.supposeTypeStatement = True
        m_oAutoSuppose.supposeTypeUser = True
        m_oAutoSuppose.loadSupposeLists m_oAdminCon
        
        'enable all features
        frmMain.ctlCM.tinyMode = False
        'load all items
        
        frmMain.ctlCM.loadCatalog _
                            m_oGlobalInfo.conMan.UID, _
                            frmMain.ctlCM.tableFilter, _
                            itemAll
                    
        'set open nodes ...
        frmMain.ctlCM.setNodeInfo m_oGlobalInfo.userProfile.expandedNodes, m_oGlobalInfo.userProfile.restoreCatalogState
    End If
            
    'check menu
    frmMain.mnuViewDirectSQL.Enabled = m_oGlobalInfo.systemtablesLoaded
    frmMain.mnuFileDisconnect.Enabled = True
    
    frmMain.chkNoTimeOut.Enabled = False
        
    
    If frmMain.mnuFileClearLRUList.Visible Then _
       frmMain.mnuFileClearLRUList.Visible = False
    
    'check for NI Connetion
    If Not oConnection.isNIConnection Then
    
        'add connection to LRU List
        m_oLRUConnections.addLRU m_oGlobalInfo.conMan.driver, _
                                 m_oGlobalInfo.conMan.UID, _
                                 m_oGlobalInfo.conMan.PWD, _
                                 m_oGlobalInfo.conMan.database, _
                                 m_oGlobalInfo.conMan.dbServer
    End If
        
    setLRUList
        
    'Show connection state
    setCompleteState "", _
                     m_oGlobalInfo.conMan.UID, _
                     m_oGlobalInfo.conMan.database, _
                     m_oGlobalInfo.conMan.dbServer
                     
    If oConnection.isNIConnection Then setTaskIDToMainWnd
    
    
    lValue = getRegistryLongValue(CATALOG_REGKEY, CATALOG_WIDTH)
    If lValue < 1 Then
        frmMain.picContainerCM.Width = frmMain.Width \ 5
    Else
        frmMain.picContainerCM.Width = lValue
    End If
    displayCatalogManager getVisible(CATALOG_REGKEY)
    
    
    lValue = getRegistryLongValue(STATUSWINDOW_REGKEY, STATUSWINDOW_HEIGTH)
    If lValue < 1 Then
        frmMain.ctlStatusWindow.Height = frmMain.Height \ 5
    Else
        frmMain.ctlStatusWindow.Height = lValue
    End If
    displayStatusWindow getVisible(STATUSWINDOW_REGKEY)
    
    
    'open a sql dialog
    If m_oGlobalInfo.systemtablesLoaded Then
        If oConnection.isNIConnection Then
            openNewSQLDialog AutoCommitOn, ""
        Else
            openNewSQLDialog m_oAdminCon.AutoCommit, ""
        End If
    End If
    
    setViewMenusForAllForms
    
    connect = True
    
    Exit Function

errConnect:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      disconnect
'    description:   disconnects sql studio from the database
'    arguments:     [out] Cancel
'
'    return value:
'
'=========================================================================================
Sub disconnect(nCancel As Integer)
    On Error GoTo errDisconnect
    
    Const FUNCTION_NAME = "disconnect"
    
    '=varibles===================================================================
    Dim lRun            As Long
    '============================================================================
        
    'exit if user is not connected
    If Not m_oGlobalInfo.conMan.isConnected Then Exit Sub
        
    'inits
    m_oGlobalInfo.disconnectCanceled = False
            
    'save the catalog list before the form unload event occurs
    frmMain.ctlCM.saveTableFilterList
              
    'unload all database connected forms
    For lRun = Forms.count - 1 To 0 Step -1
    
        If Forms(lRun).Caption <> frmMain.Caption Then
        
            Unload Forms(lRun)
            If m_oGlobalInfo.disconnectCanceled Then  'Disconnect canceled?
            
                nCancel = 1
                Exit Sub
            End If
        End If
    Next
    
    'disconnect canceled, the globalcom property can be set the user by a dialog
    'which pops up when a form is unloading
    'and the data of the form has changed
    If m_oGlobalInfo.disconnectCanceled Then
    
        nCancel = 1
        Exit Sub
    End If
    
    'if system tables are loaded
    m_oLRUConnections.savePassWord = m_oGlobalInfo.userProfile.storePassword
    
    'just call ApplyPasswordOption once, if there was a open session and not in Query_unload form
    If m_oGlobalInfo.conMan.isConnected Then
        m_oLRUConnections.applyPasswordOption
    End If
    
    'first clear catalog view
    frmMain.ctlCM.clearResults
    'then disconnect
    m_oGlobalInfo.conMan.disconnect
        
    setRegistryLongValue CATALOG_REGKEY, CATALOG_WIDTH, frmMain.picContainerCM.Width
    setRegistryLongValue STATUSWINDOW_REGKEY, STATUSWINDOW_HEIGTH, frmMain.ctlStatusWindow.Height
    
    setVisible CATALOG_REGKEY, frmMain.picContainerCM.Visible
    setVisible STATUSWINDOW_REGKEY, frmMain.ctlStatusWindow.Visible
    
    displayCatalogManager False
    displayStatusWindow False
    
       
    'Connection disabling
    frmMain.tbrMain.Buttons.item("Execute").Enabled = False
    frmMain.tbrMain.Buttons.item("SQLDialog").Enabled = False
    frmMain.tbrMain.Buttons.item("SQLDialogACOff").Enabled = False
    frmMain.tbrMain.Buttons.item("VisualQuery").Enabled = False
    frmMain.tbrMain.Buttons.item("FormDialog").Enabled = False
            
    frmMain.tbrMain.Buttons.item("Connect").Image = "Connect"
    frmMain.tbrMain.Buttons.item("Connect").ToolTipText = "Connect"
    
    frmMain.tbrMain.Buttons.item("CreateTable").Enabled = False
    frmMain.tbrMain.Buttons.item("CreateView").Enabled = False
    frmMain.tbrMain.Buttons.item("CreateSynonym").Enabled = False
    frmMain.tbrMain.Buttons.item("CreateIndex").Enabled = False
    frmMain.tbrMain.Buttons.item("CreateSequence").Enabled = False
    
    frmMain.tbrMain.Buttons.item(CATALOGMANAGER).Enabled = False
    frmMain.tbrMain.Buttons.item(STATUSWINDOW).Enabled = False
            
    If Not frmMain.mnuFileClearLRUList.Visible Then frmMain.mnuFileClearLRUList.Visible = True
    
    frmMain.mnuFileDisconnect.Enabled = False
    frmMain.mnuViewDirectSQL.Enabled = False
    frmMain.mnuViewDirectSQLAC.Enabled = False
    frmMain.mnuViewNewQueryByExample.Enabled = False
    frmMain.mnuViewNewVisualQuery.Enabled = False
            
    frmMain.mnuViewCatalogManager.Enabled = False
    frmMain.mnuViewCatalogManager.Checked = False
    frmMain.tbrMain.Buttons(CATALOGMANAGER).Enabled = False
    frmMain.tbrMain.Buttons(CATALOGMANAGER).Value = tbrUnpressed
    
    frmMain.mnuViewStatusWindow.Enabled = False
    frmMain.mnuViewStatusWindow.Checked = False
    frmMain.tbrMain.Buttons(STATUSWINDOW).Enabled = False
    frmMain.tbrMain.Buttons(STATUSWINDOW).Value = tbrUnpressed
    frmMain.picContainerSW.Height = frmMain.lblState.Height

    frmMain.mnuWindow.Enabled = False

    frmMain.chkNoTimeOut.Enabled = True
            
    Set m_oAutoSuppose = Nothing
            
    'Set disconnected state
    setCompleteState LoadResString(113), "", "", ""

    Exit Sub

errDisconnect:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      openNewSQLDialog
'    description:   opens a SQL Dialog window
'    arguments:     [out] Autocommit, flag to indicates autocommit mode for sql window
'                   [out] sText
'
'    return value:   /
'
'=========================================================================================
Sub openNewSQLDialog(AutoCommit As AutoCommit, sText As String)
    On Error GoTo errOpenNewDirectSQL
    
    Const FUNCTION_NAME = "openNewSQLDialog"
    
    '=varibles===================================================================
    Dim bConnectSuccess         As Boolean
    Dim oConnection             As SQLStudioDBLayer.connection
    Dim NewForm                 As New frmDirectSQL
    '=========================================================================
    
    'init
    bConnectSuccess = False
    
    If Forms.count > m_csMaxForm Then
    
        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If
    
    'only a not exclusive user can open another database connection
    If AutoCommit = AutoCommitOff Then
                
        If m_oGlobalInfo.conMan.isNotExclusive = False Then
        
            MsgBox LoadResString(114)
            Exit Sub
        End If
    End If
    
    'connect
    setState LoadResString(101), False
    m_oGlobalInfo.Hourglass.State = qpHourglassOn
    
    If m_oGlobalInfo.conMan.isNotExclusive And Not m_oGlobalInfo.conMan.isNIConnection Then
        
        bConnectSuccess = m_oGlobalInfo.conMan.getConnection( _
                                m_oGlobalInfo.userProfile.appManagment.isoLevel, _
                                m_oGlobalInfo.userProfile.appManagment.sqlMode, _
                                AutoCommit, _
                                oConnection)
    Else
        bConnectSuccess = m_oGlobalInfo.conMan.getConnection( _
                                m_oGlobalInfo.conMan.driverIsoLevel, _
                                m_oGlobalInfo.conMan.driverSQLMode, _
                                AutoCommit, _
                                oConnection)
    End If
    
    m_oGlobalInfo.Hourglass.State = qpHourglassOff
    
    If Not bConnectSuccess Then
    
        showCriticalConMsg oConnection
        Exit Sub
    End If
    
    'load form
    Load NewForm
    
    'set properties
    m_lSQLDialogCounter = m_lSQLDialogCounter + 1
    NewForm.Caption = LoadResString(207) + " " + Trim$(Str$(m_lSQLDialogCounter))
    
    If AutoCommit = AutoCommitOff Then
        NewForm.Icon = frmMain.IconList.ListImages(1).Picture
    End If
    
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.statementList = m_oStatementList
    Set NewForm.connection = oConnection
    
    Set NewForm.StoredFileContent = Nothing
    'set text for new sql dialog
    NewForm.Text = sText
          
    'show form
    NewForm.Show
    If NewForm.Visible Then NewForm.SetFocus
    
    NewForm.refreshFont
    
    setState "", False
    
    Exit Sub
    
errOpenNewDirectSQL:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      openNewTableDefinition
'    description:   opens a table definition window
'    arguments:     [out] sOwner, owner of an initial table
'                   [out] sTable, table name of an initial table
'
'    return value:   /
'
'=========================================================================================
Sub openNewTableDefinition(sOwner As String, sTable As String)
    On Error GoTo errOpenNewTableDefinition
    
    Const FUNCTION_NAME = "openNewTableDefinition"

    '=varibles===================================================================
    Dim bConnectSuccess         As Boolean
    Dim oConnection             As SQLStudioDBLayer.connection
    Dim NewForm                 As New frmCreateTable
    '=========================================================================

    'init
    bConnectSuccess = False

    If Forms.count > m_csMaxForm Then

        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If

    'connect
    setState LoadResString(101), False
    m_oGlobalInfo.Hourglass.State = qpHourglassOn

    If m_oGlobalInfo.conMan.isNotExclusive Then
        bConnectSuccess = m_oGlobalInfo.conMan.getConnection( _
                                    m_oGlobalInfo.userProfile.appManagment.isoLevel, _
                                    Internal, _
                                    AutoCommitOff, _
                                    oConnection)

        If Not bConnectSuccess Then
    
            showCriticalConMsg oConnection
            Exit Sub
        End If
    Else
        Exit Sub
    End If
    
    m_oGlobalInfo.Hourglass.State = qpHourglassOff

    'load form
    Load NewForm
    
    'set properties
    m_lNewTableCounter = m_lNewTableCounter + 1
    NewForm.Caption = LoadResString(215) + " " + Trim$(Str$(m_lNewTableCounter))
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.connection = oConnection
    Set NewForm.StoredFileContent = Nothing
    
    NewForm.mode = 0 'CreateNew
    
    'open the new table definition with an intial table content
    If Len(sOwner) > 0 And Len(sTable) > 0 Then
        NewForm.loadTableDefByTable sOwner, sTable
    End If
    
    'show form
    NewForm.Show
    If NewForm.Visible Then NewForm.SetFocus
    
    setState "", False
    
    Exit Sub
    
errOpenNewTableDefinition:
    m_oGlobalInfo.Hourglass.State = qpHourglassOff
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      openViewDefinition
'    description:   opens an existing view definition
'                   or a new view definition based on a table,synonym or view
'    arguments:     [in] sName  - Name of the view
'                   [in] sOwner - Owner of the view
'                   [in] sNew   - Flag if a new or existing view will be opened
'    return value:   /
'
'=========================================================================================
Sub openViewDefinition(sName As String, _
                       sOwner As String, _
                       bNew As Boolean)
    On Error GoTo errOpenViewDefinition
    
    Const FUNCTION_NAME = "openViewDefinition"

    '=varibles===================================================================
    Dim bConnectSuccess         As Boolean
    Dim oConnection             As SQLStudioDBLayer.connection
    Dim NewForm                 As New frmCreateView
    Dim sql                     As String
    '=========================================================================
    
    'init
    bConnectSuccess = False

    If Forms.count > m_csMaxForm Then
        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If

    'connect
    setState LoadResString(101), False
    
    If bNew Then
        m_oGlobalInfo.Hourglass.State = qpHourglassOn

        bConnectSuccess = m_oGlobalInfo.conMan.getConnection( _
                                m_oGlobalInfo.userProfile.appManagment.isoLevel, _
                                Internal, _
                                AutoCommitOff, _
                                oConnection)

        m_oGlobalInfo.Hourglass.State = qpHourglassOff

        If Not bConnectSuccess Then

            showCriticalConMsg oConnection
            Exit Sub
        End If
    Else
        'if view def is shown, open a standard commit on connection
        bConnectSuccess = m_oGlobalInfo.conMan.getConnection( _
                                                m_oGlobalInfo.conMan.driverIsoLevel, _
                                                m_oGlobalInfo.conMan.driverSQLMode, _
                                                AutoCommitOn, _
                                                oConnection)
        If Not bConnectSuccess Then
    
            showCriticalConMsg oConnection
            Exit Sub
        End If
    
        'check if required data is available
        sql = "SELECT definition FROM domain.viewdefs WHERE viewname = '" + m_oComFunc.convertStringForCharCol(sName, True) + "' AND owner = '" + m_oComFunc.convertStringForCharCol(sOwner, True) + "'"
        If checkResult(sql) = False Then
            MsgBox LoadResString(226), vbCritical, LoadResString(230)
            Exit Sub
        End If
    End If
    
    'load form
    Load NewForm
    
    'set properties
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.connection = oConnection
    Set NewForm.StoredFileContent = Nothing
    
    NewForm.appVersion = CLng(SQLSTOBJECTSVERSION)
    
    If bNew = True Then
        m_lNewViewCounter = m_lNewViewCounter + 1
        NewForm.Caption = LoadResString(218) + " " + Trim$(Str$(m_lNewViewCounter))
        NewForm.loadTableDefinition sOwner, sName
    Else
        NewForm.Caption = LoadResString(222) + " - " + sOwner + "." + sName
        NewForm.loadViewDefinition sName, sOwner
    End If
    
    'show form
    NewForm.Show
    If NewForm.Visible Then NewForm.SetFocus
    
    setState "", False
    
    Exit Sub
    
errOpenViewDefinition:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      openNewViewDefinition
'    description:   opens a new view definition defined by a stored sql studio object
'    arguments:     [in] oSFC, storedFileContent
'    return value:   /
'
'=========================================================================================
Sub openNewViewDefinition(oSFC As StoredFileContent)
    On Error GoTo errOpenNewViewDefinition
    
    Const FUNCTION_NAME = "openNewViewDefinition"

    '=varibles===================================================================
    Dim bConnectSuccess         As Boolean
    Dim oConnection             As SQLStudioDBLayer.connection
    Dim NewForm                 As New frmCreateView
    Dim sql                     As String
    '=========================================================================
    
    'init
    bConnectSuccess = False

    If Forms.count > m_csMaxForm Then
        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If

    'connect
    setState LoadResString(101), False

    m_oGlobalInfo.Hourglass.State = qpHourglassOn

    bConnectSuccess = m_oGlobalInfo.conMan.getConnection( _
                            m_oGlobalInfo.userProfile.appManagment.isoLevel, _
                            Internal, _
                            AutoCommitOff, _
                            oConnection)

    m_oGlobalInfo.Hourglass.State = qpHourglassOff

    If Not bConnectSuccess Then

        showCriticalConMsg oConnection
        Exit Sub
    End If
    
    
    'load form
    Load NewForm
    
    'set properties
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.connection = oConnection
    Set NewForm.StoredFileContent = Nothing
    
    NewForm.appVersion = CLng(SQLSTOBJECTSVERSION)
    
    m_lNewViewCounter = m_lNewViewCounter + 1
    NewForm.Caption = LoadResString(218) + " " + Trim$(Str$(m_lNewViewCounter))
    If Not oSFC Is Nothing Then
        NewForm.loadQueryDefByStoredObject oSFC
    End If
    
    'show form
    NewForm.Show
    If NewForm.Visible Then NewForm.SetFocus
    
    setState "", False
    
    Exit Sub
    
errOpenNewViewDefinition:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      openIndexDefinition
'    description:   opens an existing index definition
'    arguments:     [in] sName  - Name of the index
'                   [in] sOwner - Owner of the index
'                   [in] bNew   - Flag if a new or existing index will be opened
'    return value:   /
'
'=========================================================================================
Sub openIndexDefinition(sName As String, _
                        sOwner As String, _
                        sTable As String, _
                        bNew As Boolean)
    On Error GoTo errOpenIndexDefinition
    
    Const FUNCTION_NAME = "openIndexDefinition"

    '=varibles===================================================================
    Dim bConnectSuccess         As Boolean
    Dim oConnection             As SQLStudioDBLayer.connection
    Dim NewForm                 As New frmCreateIndex
    Dim sql                     As String
    '=========================================================================

    'init
    bConnectSuccess = False

    If Forms.count > m_csMaxForm Then
    
        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If

    'connect
    setState LoadResString(101), False

    If bNew Then
    
        m_oGlobalInfo.Hourglass.State = qpHourglassOn
        
        bConnectSuccess = m_oGlobalInfo.conMan.getConnection( _
                                m_oGlobalInfo.userProfile.appManagment.isoLevel, _
                                Internal, _
                                AutoCommitOff, _
                                oConnection)

        m_oGlobalInfo.Hourglass.State = qpHourglassOff
        
        If Not bConnectSuccess Then
    
            showCriticalConMsg oConnection
            Exit Sub
        End If
    Else
        'if view def is shown, open a standard commit on connection
        bConnectSuccess = m_oGlobalInfo.conMan.getConnection( _
                                                m_oGlobalInfo.conMan.driverIsoLevel, _
                                                m_oGlobalInfo.conMan.driverSQLMode, _
                                                AutoCommitOn, _
                                                oConnection)
        If Not bConnectSuccess Then
    
            showCriticalConMsg oConnection
            Exit Sub
        End If
    End If
        

    If Not bNew Then
    
        'check if required data is available
        sql = "SELECT columnname, sort FROM domain.indexcolumns WHERE indexname = '" + m_oComFunc.convertStringForCharCol(sName, True) + "' AND owner = '" + m_oComFunc.convertStringForCharCol(sOwner, True) + "'" + " AND TABLENAME ='" + m_oComFunc.convertStringForCharCol(sTable, True) + "'"
        If checkResult(sql) = False Then
            MsgBox LoadResString(227), vbCritical, LoadResString(230)
            Exit Sub
        End If
    End If
    
    'load form
    Load NewForm
    
    'set properties
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.connection = oConnection
    NewForm.appVersion = CLng(SQLSTOBJECTSVERSION)
    
    If bNew = True Then
        m_lNewIndexCounter = m_lNewIndexCounter + 1
        NewForm.Caption = LoadResString(219) + " " + Trim$(Str$(m_lNewIndexCounter))
        If Len(sOwner) > 0 And Len(sName) > 0 Then
            NewForm.loadColumnDefinition sOwner, sName
        End If
    Else
        NewForm.Caption = LoadResString(223) + " - " + sOwner + "." + sName
        NewForm.loadIndexDefinition sName, sOwner, sTable
    End If
    
    'show form
    NewForm.Show
    If NewForm.Visible Then NewForm.SetFocus
    
    setState "", False
    
    Exit Sub
    
errOpenIndexDefinition:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      openSynonymDefinition
'    description:   opens an existing synonym definition
'    arguments:     [in] sName  - Name of the synonym
'                   [in] sOwner - Owner of the synonym
'                   [in] sNew   - Flag if a new or existing synonym will be opened
'    return value:   /
'
'=========================================================================================
Sub openSynonymDefinition(sName As String, sOwner As String, bNew As Boolean)
    On Error GoTo errOpenSynonymDefinition
    
    Const FUNCTION_NAME = "openSynonymDefinition"

    '=variables===================================================================
    Dim bConnectSuccess         As Boolean
    Dim oConnection             As SQLStudioDBLayer.connection
    Dim NewForm                 As New frmCreateSynonym
    Dim sql                     As String
    '=========================================================================

    'init
    bConnectSuccess = False

    If Forms.count > m_csMaxForm Then
        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If

    'connect
    setState LoadResString(101), False

    If bNew Then
    
        m_oGlobalInfo.Hourglass.State = qpHourglassOn
        bConnectSuccess = m_oGlobalInfo.conMan.getConnection( _
                                    m_oGlobalInfo.userProfile.appManagment.isoLevel, _
                                    Internal, _
                                    AutoCommitOff, _
                                    oConnection)

        m_oGlobalInfo.Hourglass.State = qpHourglassOff
        
        If Not bConnectSuccess Then
    
            showCriticalConMsg oConnection
            Exit Sub
        End If
    Else
        'if view def is shown, open a standard commit on connection
        bConnectSuccess = m_oGlobalInfo.conMan.getConnection( _
                                                m_oGlobalInfo.conMan.driverIsoLevel, _
                                                m_oGlobalInfo.conMan.driverSQLMode, _
                                                AutoCommitOn, _
                                                oConnection)
        If Not bConnectSuccess Then
    
            showCriticalConMsg oConnection
            Exit Sub
        End If
    End If
        
    If Not bNew Then
    
        'check if required data is available
        sql = "SELECT tablename, public, comment FROM domain.synonyms WHERE synonymname = '" + m_oComFunc.convertStringForCharCol(sName, True) + "' AND owner = '" + m_oComFunc.convertStringForCharCol(sOwner, True) + "'"
        If checkResult(sql) = False Then
            MsgBox LoadResString(228), vbCritical, LoadResString(230)
            Exit Sub
        End If
    End If
    
    'load form
    Load NewForm
    
    'set properties
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.connection = oConnection
    NewForm.appVersion = CLng(SQLSTOBJECTSVERSION)
    
    If bNew = True Then
        m_lNewSynonymCounter = m_lNewSynonymCounter + 1
        NewForm.Caption = LoadResString(220) + " " + Trim$(Str$(m_lNewSynonymCounter))
        If Len(sName) > 0 Then
            NewForm.dragDropActions sName
        End If
    Else
        NewForm.Caption = LoadResString(224) + " - " + sOwner + "." + sName
        NewForm.loadSynonymDefinition sName, sOwner
    End If
    
    'show form
    NewForm.Show
    If NewForm.Visible Then NewForm.SetFocus
    
    setState "", False
    
    Exit Sub
    
errOpenSynonymDefinition:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      openSequenceDefinition
'    description:   opens an existing sequence definition
'    arguments:     [in] sName  - Name of the sequence
'                   [in] sOwner - Owner of the sequence
'                   [in] sNew   - Flag if a new or existing sequence will be opened
'    return value:   /
'
'=========================================================================================
Sub openSequenceDefinition(sName As String, sOwner As String, bNew As Boolean)
    On Error GoTo errOpenSequenceDefinition
    
    Const FUNCTION_NAME = "openSequenceDefinition"

    '=varibles===================================================================
    Dim bConnectSuccess         As Boolean
    Dim oConnection             As SQLStudioDBLayer.connection
    Dim NewForm                 As New frmCreateSequence
    Dim sql                     As String
    '=========================================================================

    'init
    bConnectSuccess = False

    If Forms.count > m_csMaxForm Then
        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If

    'connect
    setState LoadResString(101), False

    If bNew Then
    
        m_oGlobalInfo.Hourglass.State = qpHourglassOn
        
        bConnectSuccess = m_oGlobalInfo.conMan.getConnection( _
                                m_oGlobalInfo.userProfile.appManagment.isoLevel, _
                                Internal, _
                                AutoCommitOff, _
                                oConnection)

        m_oGlobalInfo.Hourglass.State = qpHourglassOff
        If Not bConnectSuccess Then
    
            showCriticalConMsg oConnection
            Exit Sub
        End If
    Else
        'if view def is shown, open a standard commit on connection
        bConnectSuccess = m_oGlobalInfo.conMan.getConnection( _
                                                m_oGlobalInfo.conMan.driverIsoLevel, _
                                                m_oGlobalInfo.conMan.driverSQLMode, _
                                                AutoCommitOn, _
                                                oConnection)
        If Not bConnectSuccess Then
    
            showCriticalConMsg oConnection
            Exit Sub
        End If
    End If
    
    If Not bNew Then
    
        'check if required data is available
        sql = "SELECT min_value, max_value, increment_by, cycle_flag, order_flag, cache_size, comment FROM domain.sequences WHERE sequence_name = '" + m_oComFunc.convertStringForCharCol(sName, True) + "' AND owner = '" + m_oComFunc.convertStringForCharCol(sOwner, True) + "'"
        If checkResult(sql) = False Then
            MsgBox LoadResString(229), vbCritical, LoadResString(230)
            Exit Sub
        End If
    End If
    
    'load form
    Load NewForm
    
    'set properties
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.connection = oConnection
    NewForm.appVersion = CLng(SQLSTOBJECTSVERSION)
    If bNew = True Then
        m_lNewSeqCounter = m_lNewSeqCounter + 1
        NewForm.Caption = LoadResString(221) + " " + Trim$(Str$(m_lNewSeqCounter))
    Else
        NewForm.Caption = LoadResString(225) + " - " + sOwner + "." + sName
        NewForm.loadSequenceDefinition sName, sOwner
    End If
    
    'show form
    NewForm.Show
    If NewForm.Visible Then NewForm.SetFocus
    
    setState "", False
    
    Exit Sub
    
errOpenSequenceDefinition:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      showDirectSQLAutoCommitOff
'    description:   opens a dsql dialog window, in autocommit off mode
'    arguments:     oConnection, connection object
'
'    return value:   /
'
'=========================================================================================
Sub showDirectSQLAutoCommitOff(oConnection As SQLStudioDBLayer.connection)
    On Error GoTo errShowDirectSQLAutoCommitOff
    
    Const FUNCTION_NAME = "showDirectSQLAutoCommitOff"
        
    '=varibles===================================================================
    Dim NewForm                 As New frmDirectSQL
    '=========================================================================
    
    If Forms.count > m_csMaxForm Then
    
        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If
    
    'load form
    Load NewForm
    
    'set properties
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.connection = oConnection
    Set NewForm.StoredFileContent = Nothing
    
    NewForm.mnuViewDirectSQL.Enabled = m_oGlobalInfo.systemtablesLoaded
    NewForm.mnuViewDirectSQLAC.Enabled = m_oGlobalInfo.systemtablesLoaded
    NewForm.mnuViewNewQueryByExample.Enabled = m_oGlobalInfo.systemtablesLoaded
    NewForm.mnuViewNewVisualQuery.Enabled = m_oGlobalInfo.systemtablesLoaded
      
    'show form
    NewForm.Show
    
    If m_oGlobalInfo.systemtablesLoaded = False Then removeCloseButton NewForm
        
    If NewForm.Visible Then NewForm.SetFocus
        
    setState "", False
    
    Exit Sub
    
errShowDirectSQLAutoCommitOff:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      openNewVisualQuery
'    description:   shows visual query dialog
'    arguments:     [out] aTableObject
'
'    return value:  /
'
'=========================================================================================
Public Function openNewVisualQuery(Optional aTableObject As TableObject)
    On Error GoTo errOpenNewVisualQuery
    
    Const FUNCTION_NAME = "openNewVisualQuery"
    
    '=varibles===================================================================
    Dim bConnectSuccess         As Boolean
    Dim oConnection             As SQLStudioDBLayer.connection
    Dim NewForm                 As New frmVisualQuery
    '=========================================================================
    
    If Forms.count > m_csMaxForm Then
    
        MsgBox LoadResString(115), vbCritical
        Exit Function
    End If
    
    bConnectSuccess = m_oGlobalInfo.conMan.getConnection( _
                            m_oGlobalInfo.userProfile.appManagment.isoLevel, _
                            Internal, _
                            AutoCommitOn, oConnection)

    If Not bConnectSuccess Then
    
        setState LoadResString(100), True
        Exit Function
    End If
    
    'load form
    Load NewForm
    
    'set properties
    Set NewForm.globalInfo = m_oGlobalInfo
    
    'set new object description
    m_lVIQCounter = m_lVIQCounter + 1
    NewForm.Caption = LoadResString(213) + " " + Trim$(Str$(m_lVIQCounter))
    Set NewForm.connection = oConnection
    Set NewForm.StoredFileContent = Nothing
    
    If IsMissing(aTableObject) = False Then
        'set a table object in new dialog
        NewForm.setDBOTable aTableObject
    End If
          
    'show form
    NewForm.Show
    
    Exit Function
    
errOpenNewVisualQuery:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      openNewFormDialog
'    description:   shows Form Dialog dialog window
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Sub openNewFormDialog(Optional aTableObject As TableObject)
    On Error GoTo ErrOpenNewQueryByExample
    
    Const FUNCTION_NAME = "openNewFormDialog"

    '=variables========================================================================
    Dim bConnectSuccess         As Boolean
    Dim oConnection             As SQLStudioDBLayer.connection
    Dim NewForm                 As New frmQuickQBE
    '=========================================================================
    
    If Forms.count > m_csMaxForm Then
    
        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If
    
    bConnectSuccess = m_oGlobalInfo.conMan.getConnection( _
                        m_oGlobalInfo.userProfile.appManagment.isoLevel, _
                        Internal, _
                        AutoCommitOn, oConnection)
                        
    If Not bConnectSuccess Then
    
        setState LoadResString(100), True
        Exit Sub
    End If
    
    'Load form
    Load NewForm
    
    m_lFormDialogCounter = m_lFormDialogCounter + 1
    NewForm.Caption = LoadResString(214) + " " + Trim$(Str$(m_lFormDialogCounter))
    
    'Set properties
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.connection = oConnection
    Set NewForm.StoredFileContent = Nothing
    
     If IsMissing(aTableObject) = False Then
        'set a table object in new dialog
        NewForm.setDBOTable aTableObject
    End If
    
    NewForm.Show 'Show form
    
    Exit Sub
    
ErrOpenNewQueryByExample:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      openSQLDialog
'    description:   shows SQL Dialog dialog window
'    arguments:     [in]  oSFC, sql studio storedFileContent object
'
'    return value:  /
'
'=========================================================================================
Sub openSQLDialog(oSFC As StoredFileContent)
    On Error GoTo ErropenSQLDialog
    
    Const FUNCTION_NAME = "openSQLDialog"
    
    '=varibles===================================================================
    Dim bConnectSuccess      As Boolean
    Dim oConnection          As SQLStudioDBLayer.connection
    Dim NewForm              As New frmDirectSQL
    '=========================================================================
    
    If Forms.count > m_csMaxForm Then
    
        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If
    
    If oSFC.AutoCommit = AutoCommitOff Then
    
        If m_oGlobalInfo.conMan.isNotExclusive = False Then _
           MsgBox LoadResString(114)
    End If
    
    m_oGlobalInfo.Hourglass.State = qpHourglassOn
    bConnectSuccess = m_oGlobalInfo.conMan.getConnection( _
                                            oSFC.isoLevel, _
                                            oSFC.sqlMode, _
                                            oSFC.AutoCommit, _
                                            oConnection)
                                             
    m_oGlobalInfo.Hourglass.State = qpHourglassOff
    
    If Not bConnectSuccess Then
    
        showCriticalConMsg oConnection
        Exit Sub
    End If
      
    'Load form
    Load NewForm
    
    'Set properties
    If oSFC.AutoCommit = AutoCommitOff Then
        NewForm.Icon = frmMain.IconList.ListImages(1).Picture
    End If
    
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.statementList = m_oStatementList
    Set NewForm.connection = oConnection
    '----------------------------------------------------
    'init the Connection of the storedFileContent connection too
    '----------------------------------------------------
    Set NewForm.StoredFileContent = oSFC
    
    'now set the dialog caption
    If Len(oSFC.Filename) < 1 Then
    
        m_lSQLDialogCounter = m_lSQLDialogCounter + 1
        NewForm.Caption = LoadResString(207) + " " + Trim$(Str$(m_lSQLDialogCounter))
    End If
    
    'Show form
    NewForm.Show
    
    NewForm.refreshFont
    
    Exit Sub
    
ErropenSQLDialog:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      openVisualQuery
'    description:   shows viusal query dialog window
'    arguments:     [in]  oSFC, sql studio storedFileContent object
'
'    return value:  /
'
'=========================================================================================
Sub openVisualQuery(oSFC As StoredFileContent)
    On Error GoTo ErrOpenVisualQuery
    
    Const FUNCTION_NAME = "openVisualQuery"
    
    '=varibles===================================================================
    Dim bConnectSuccess      As Boolean
    Dim oConnection          As SQLStudioDBLayer.connection
    Dim NewForm              As New frmVisualQuery
    '=========================================================================
    
    If Forms.count > m_csMaxForm Then
    
        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If
    
    m_oGlobalInfo.Hourglass.State = qpHourglassOn
    
    
    bConnectSuccess = m_oGlobalInfo.conMan.getConnection( _
                      oSFC.isoLevel, _
                      oSFC.sqlMode, _
                      AutoCommitOn, oConnection)
                      
    m_oGlobalInfo.Hourglass.State = qpHourglassOff
    
    If Not bConnectSuccess Then
    
        showCriticalConMsg oConnection
        Exit Sub
    End If
      
    'Load form
    Load NewForm
    
    'Set properties
    Set NewForm.connection = oConnection
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.StoredFileContent = oSFC
    
    'if file is not a stored viq
    If Len(oSFC.Filename) < 1 Then
        'set new object description
        m_lVIQCounter = m_lVIQCounter + 1
        NewForm.Caption = LoadResString(213) + " " + Trim$(Str$(m_lVIQCounter))
    End If

      
    'Show form
    NewForm.Show
       
    Exit Sub
    
ErrOpenVisualQuery:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      openFormDialog
'    description:   shows Form Dialog dialog window
'    arguments:     [in]  oSFC, sql studio storedFileContent object
'
'    return value:  /
'
'=========================================================================================
Sub openFormDialog(oSFC As StoredFileContent)
    On Error GoTo ErrOpenQueryByExample
    
    Const FUNCTION_NAME = "openFormDialog"
       
    '=varibles===================================================================
    Dim bConnectSuccess      As Boolean
    Dim oConnection          As SQLStudioDBLayer.connection
    Dim NewForm              As New frmQuickQBE
    '=========================================================================
    
    If Forms.count > m_csMaxForm Then
    
        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If
    
    m_oGlobalInfo.Hourglass.State = qpHourglassOn
    
    bConnectSuccess = m_oGlobalInfo.conMan.getConnection( _
                      oSFC.isoLevel, _
                      oSFC.sqlMode, _
                      AutoCommitOn, oConnection)
                  
    m_oGlobalInfo.Hourglass.State = qpHourglassOff
    
    If Not bConnectSuccess Then
    
        showCriticalConMsg oConnection
        Exit Sub
    End If
    
    'Load form
    Load NewForm
           
    
    'Set properties
    Set NewForm.connection = oConnection
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.StoredFileContent = oSFC
    
    If Len(oSFC.Filename) < 1 Then
        m_lFormDialogCounter = m_lFormDialogCounter + 1
        NewForm.Caption = LoadResString(214) + " " + Trim$(Str$(m_lFormDialogCounter))
    End If
      
    'Show form
    NewForm.Show
    
    Exit Sub
    
ErrOpenQueryByExample:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      displayStatusWindow
'    description:   opens the sql status form
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Sub displayStatusWindow(bVisible As Boolean)
    On Error GoTo errdisplayStatusWindow
    
    Const FUNCTION_NAME = "displayStatusWindow"
           
    frmMain.mnuViewStatusWindow.Checked = bVisible
    frmMain.ctlStatusWindow.Visible = bVisible
        
    If (bVisible) Then
        frmMain.picContainerSW.Height = frmMain.lblState.Height + frmMain.ctlStatusWindow.Height + 8 * Screen.TwipsPerPixelY
        frmMain.picContainerSW.BorderStyle = vbFixedSingle
        frmMain.tbrMain.Buttons.item(STATUSWINDOW).Value = tbrPressed
        If frmMain.ctlStatusWindow.Visible Then frmMain.ctlStatusWindow.SetFocus
    Else
        frmMain.picContainerSW.Height = frmMain.lblState.Height
        frmMain.picContainerSW.BorderStyle = vbBSNone
        frmMain.tbrMain.Buttons.item(STATUSWINDOW).Value = tbrUnpressed
    End If

    setViewMenusForAllForms
    
    Exit Sub
    
errdisplayStatusWindow:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      displayCatalogManager
'    description:   opens the catalog form view
'    arguments:     [in] bVisible
'
'    return value:  /
'
'=========================================================================================
Sub displayCatalogManager(bVisible As Boolean)
    On Error GoTo errDisplayCatalogManager
    
    Const FUNCTION_NAME = "displayCatalogManager"
    
    frmMain.mnuViewCatalogManager.Checked = bVisible
    frmMain.picContainerCM.Visible = bVisible
    frmMain.ctlCM.Visible = bVisible
        
    If (bVisible) Then
        frmMain.tbrMain.Buttons.item(CATALOGMANAGER).Value = tbrPressed
        If frmMain.ctlCM.Visible Then frmMain.ctlCM.SetFocus
    Else
        frmMain.tbrMain.Buttons.item(CATALOGMANAGER).Value = tbrUnpressed
    End If
    
    setViewMenusForAllForms
    
    Exit Sub
    
errDisplayCatalogManager:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      openAlterTableDefinition
'    description:   shows an alter table definition dialog window
'    arguments:     [in]  tablename, tablename if dialog is used for altering a table
'                   [in]  tableowner, tableowner if dialog is used for altering a table
'                   [in]  itemType
'
'    return value:  /
'
'=========================================================================================
Sub openAlterTableDefinition(tablename As String, _
                             tableowner As String, _
                             itemType As cvItemType)
    On Error GoTo errOpenAlterTableDefinition
    
    Const FUNCTION_NAME = "openAlterTableDefinition"
    
    '=varibles===================================================================
    Dim bConnectSuccess    As Boolean
    Dim oConnection        As SQLStudioDBLayer.connection
    Dim NewForm            As New frmCreateTable
    Dim isoLevel           As SQLStudioDBLayer.isoLevel
    Dim sqlMode            As SQLStudioDBLayer.sqlMode
    '=============================================================================
    
    If Forms.count > m_csMaxForm Then
    
        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If

    m_oGlobalInfo.Hourglass.State = qpHourglassOn
    
    
    isoLevel = Committed
    sqlMode = Internal
    
    
    bConnectSuccess = m_oGlobalInfo.conMan.getConnection( _
                                            isoLevel, _
                                            sqlMode, _
                                            AutoCommitOff, _
                                            oConnection)
    If Not bConnectSuccess Then

        showCriticalConMsg oConnection
        Exit Sub
    End If
    
    m_oGlobalInfo.Hourglass.State = qpHourglassOff
      
    'Load form
    Load NewForm
    
    'Alter Table
    NewForm.mode = 3 ' AlterTable
    m_lAlterTableCounter = m_lAlterTableCounter + 1
    NewForm.Caption = LoadResString(216) + " " + Trim$(Str$(m_lAlterTableCounter))
    
    Set NewForm.connection = oConnection
        
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.StoredFileContent = Nothing
    NewForm.InitializeAlterTable tablename, tableowner, itemType
    
    'Show form
    NewForm.Show
       
    Exit Sub
    
errOpenAlterTableDefinition:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      openStoredTableDefinition
'    description:   shows table definition dialog window , for alteration
'    arguments:     [in]  oSFC, sql studio storedFileContent object
'
'    return value:  /
'
'=========================================================================================
Sub openStoredTableDefinition(oSFC As StoredFileContent)
    On Error GoTo erropenStoredTableDefinition
    
    Const FUNCTION_NAME = "openStoredTableDefinition"
    
    '=varibles===================================================================
    Dim bConnectSuccess    As Boolean
    Dim oConnection        As SQLStudioDBLayer.connection
    Dim NewForm            As New frmCreateTable
    Dim isoLevel           As SQLStudioDBLayer.isoLevel
    Dim sqlMode            As SQLStudioDBLayer.sqlMode
    '=============================================================================
    
    If Forms.count > m_csMaxForm Then
    
        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If

    m_oGlobalInfo.Hourglass.State = qpHourglassOn
    
    isoLevel = oSFC.isoLevel
    sqlMode = oSFC.sqlMode
    
    
    bConnectSuccess = m_oGlobalInfo.conMan.getConnection( _
                                            isoLevel, _
                                            sqlMode, _
                                            AutoCommitOff, _
                                            oConnection)
    If Not bConnectSuccess Then

        showCriticalConMsg oConnection
        Exit Sub
    End If
    
    m_oGlobalInfo.Hourglass.State = qpHourglassOff
      
    'Load form
    Load NewForm
    Set NewForm.connection = oConnection
    
    'Create Table
    NewForm.mode = 0 'CreateNew
    
    Set NewForm.globalInfo = m_oGlobalInfo
    'if file is a stored table draft set stored file object
    If Len(oSFC.Filename) > 0 Then
        Set NewForm.StoredFileContent = oSFC
    Else
        'if stored file content has not valid name
        'initialize the table dialog with it and open it as a new table def dialog
        m_lNewTableCounter = m_lNewTableCounter + 1
        NewForm.Caption = LoadResString(215) + " " + Trim$(Str$(m_lNewTableCounter))
        Set NewForm.StoredFileContent = Nothing
        NewForm.loadTableDefByStoredQuery oSFC
    End If
    
    'Show form
    NewForm.Show
       
    Exit Sub
    
erropenStoredTableDefinition:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      showTableDefinition
'    description:   shows table definition dialog window , for export to DirectSQL
'    arguments:     [in]  tablename, tablename if dialog is used for altering a table
'                   [in]  tableowner, tableowner if dialog is used for altering a table
'                   [in]  itemType
'
'    return value:  /
'
'=========================================================================================
Sub showTableDefinition(sName As String, _
                        sOwner As String, _
                        itemType As cvItemType)
    On Error GoTo errShowTableDefinition
    
    Const FUNCTION_NAME = "showTableDefinition"
    
    '=varibles===================================================================
    Dim bConnectSuccess    As Boolean
    Dim oConnection        As SQLStudioDBLayer.connection
    Dim NewForm            As New frmCreateTable
    Dim isoLevel           As SQLStudioDBLayer.isoLevel
    Dim sqlMode            As SQLStudioDBLayer.sqlMode
    '=============================================================================
    
    If Forms.count > m_csMaxForm Then
    
        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If

    m_oGlobalInfo.Hourglass.State = qpHourglassOn
    
    
    isoLevel = Committed
    sqlMode = Internal
        'if table def is just shown, open a standard commit on connection
        bConnectSuccess = m_oGlobalInfo.conMan.getConnection( _
                                                m_oGlobalInfo.conMan.driverIsoLevel, _
                                                m_oGlobalInfo.conMan.driverSQLMode, _
                                                AutoCommitOn, _
                                                oConnection)
        If Not bConnectSuccess Then
    
            showCriticalConMsg oConnection
            Exit Sub
        End If
    
    m_oGlobalInfo.Hourglass.State = qpHourglassOff
      
    'Load form
    Load NewForm
    
   'Create Table
    NewForm.mode = 0 'CreateNew
    
    Set NewForm.connection = oConnection
    
    'Set properties
    NewForm.Caption = LoadResString(231) + " - " + sOwner + "." + sName
    
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.StoredFileContent = Nothing
      
    
    NewForm.InitializeShowTable sName, sOwner
    
    'Show form
    NewForm.Show
       
    Exit Sub
    
errShowTableDefinition:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      setCompleteState
'    description:   set status bar and main frame caption
'    arguments:     [in] State, connection state
'                   [in] DSN, data source of odbc driver
'                   [in] UID, user name
'                   [in] Database, database name
'                   [in] dbServer, server node name
'    return value:  /
'
'=========================================================================================
Public Function setCompleteState(ByVal State As String, _
                            ByVal UID As String, _
                            ByVal database As String, _
                            ByVal dbServer As String)
    On Error GoTo errSetCompleteState
    
    Const FUNCTION_NAME = "setCompleteState"
    
    frmMain.lblState.Panels("xState").Text = State
    frmMain.lblState.Panels("xUID").Text = UID
    frmMain.lblState.Panels("xDataBase").Text = database
    
    If Not m_oGlobalInfo.conMan.isNIConnection Then
    
        If Len(dbServer) < 1 Then
            
            If Len(UID) > 0 Then
            
                dbServer = LoadResString(320)
            Else
                dbServer = ""
            End If
        End If
    Else
        dbServer = ""
    End If
    
    frmMain.lblState.Panels("xDBServer").Text = dbServer
    
    frmMain.Caption = LoadResString(102) + " ["
    frmMain.Caption = frmMain.Caption + UID + "," + database
    
    'check for NI Connetion
    If Not m_oGlobalInfo.conMan.isNIConnection Then _
       frmMain.Caption = frmMain.Caption + "," + dbServer
    
    frmMain.Caption = frmMain.Caption + "]"
    
    'check for empty caption
    If frmMain.Caption = LoadResString(102) + " [,,,]" Then frmMain.Caption = LoadResString(102)
    If frmMain.Caption = LoadResString(102) + " [,,]" Then frmMain.Caption = LoadResString(102)
    If frmMain.Caption = LoadResString(102) + " [,]" Then frmMain.Caption = LoadResString(102)
    
    Exit Function
    
errSetCompleteState:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      setState
'    description:   set status bar text
'    arguments:     [in] sState,
'                   [in] bWithTimer, delete message after a period
'    return value:  /
'
'=========================================================================================
Public Function setState(ByVal sState As String, ByVal bWithTimer As Boolean)
    On Error GoTo errSetState
    
    Const FUNCTION_NAME = "setState"
    
    frmMain.lblState.Panels("xState").Text = sState
    
    If bWithTimer Then frmMain.tmrState.Enabled = True
    
    Exit Function
    
errSetState:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      showHelp
'    description:   set status bar text
'    arguments:     [in] sHelpString, path for help (.chm) file
'
'    return value:  /
'
'=========================================================================================
Public Function showHelp(ByVal sHelpString As String)
    On Error GoTo errShowHelp
    
    Const FUNCTION_NAME = "showHelp"
    
    m_oHelp.ShowTopic sHelpString
    
    Exit Function
    
errShowHelp:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      loadAndSetLRUList
'    description:   load LRU list from registry and sets list to frmmain menu
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Function loadAndSetLRUList()
    On Error GoTo errLoadAndSetLRUList

    Const FUNCTION_NAME = "loadAndSetLRUList"
    
    m_oLRUConnections.loadConnections
    
    setLRUList
    
    Exit Function

errLoadAndSetLRUList:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      setLRUList
'    description:   sets LRU list to frmmain menu
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Function setLRUList()
    On Error GoTo errSetLRUList
    
    Const FUNCTION_NAME = "setLRUList"

    '=variables===================================================================
    Dim i        As Integer
    Dim bReturn  As Boolean
    Dim sCaption As String
    Dim sPWD     As String
    Dim sText    As String
    '=============================================================================
            
    For i = 0 To 9
    
          bReturn = m_oLRUConnections.getCaption(i, sCaption)
          If bReturn Then
          
             m_oLRUConnections.getPWD i, sPWD
          
             frmMain.mnuFileSep1.Visible = True
             frmMain.mnuLRUConnect(i).Caption = sCaption
             frmMain.mnuLRUConnect(i).Visible = True
          Else
            frmMain.mnuLRUConnect(i).Caption = ""
            frmMain.mnuLRUConnect(i).Visible = False
          End If
    Next i
    
    If m_oLRUConnections.count = 0 Then frmMain.mnuFileSep1.Visible = False
         
    Exit Function
    
errSetLRUList:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      showResults
'    description:   shows a result dialog window
'    arguments:     [in] Results, collection of result objects
'                   [in] sCaption, caption for result window
'                   [in] emResultMode, mode for result objects
'                   [in] sTimeStart, beginning of action
'                   [in] sTimeEnd, end of action
'
'    return value:  true, if successfully
'
'=========================================================================================
Public Function showResults(results As Collection, _
                            sCaption As String, _
                            emResultMode As SQLStudioDBLayer.sapdbResultMode, _
                            sTimeStart As String, _
                            sTimeEnd As String _
                            ) As Boolean
    On Error GoTo errShowResults
    
    Const FUNCTION_NAME = "showResults"
    
    '=varibles===================================================================
    Dim lRun       As Long
    Dim NewForm    As frmShowResult
    '=============================================================================
  
    'rc
    showResults = False
    
    'Is there an result?
    If results Is Nothing Then
    
        showResults = True
        Exit Function
    End If
    
    If results.count > 0 Then

        Set NewForm = New frmShowResult
        Set NewForm.globalInfo = m_oGlobalInfo
        
        NewForm.resultMode = emResultMode
        Set NewForm.results = results
        
        'http://pts:1080/webpts?wptsdetail=yes&ErrorType=1&ErrorID=1122089
        If Len(sCaption) > 40 Then sCaption = Left(sCaption, 37) + "..."
        
        NewForm.Caption = sCaption
        NewForm.showResult sTimeStart, sTimeEnd
    End If
    
    'Set rc
    showResults = True

    Exit Function

errShowResults:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      showContent
'    description:   show the content of the object
'    arguments:     [in]  sOwner, owner of the object
'                   [in]  sName, name of the object
'
'    return value:  /
'
'=========================================================================================
Public Function showContent(sOwner As String, sName As String) As Boolean
    On Error GoTo errShowContent

    Const FUNCTION_NAME = "showContent"
   
    showContent = False
    
    executeStatement "SELECT * FROM " + Chr$(34) + sOwner + Chr$(34) + "." + Chr$(34) + sName + Chr$(34)

    showContent = True

    Exit Function

errShowContent:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
Public Function showUserGroup(sOwner As String, sUserGroup As String) As Boolean
    On Error GoTo errShowUserGroup

    Const FUNCTION_NAME = "showUserGroup"
   
    'Init
    showUserGroup = False
    
    executeStatement "SELECT * FROM USERS WHERE OWNER ='" + m_oComFunc.convertStringForCharCol(sOwner, True) + "' AND GROUPNAME ='" + m_oComFunc.convertStringForCharCol(sUserGroup, True) + "' AND USERNAME = ''"
                    
    showUserGroup = True

    Exit Function

errShowUserGroup:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function

Public Function showUser(sOwner As String, sUser As String) As Boolean
    On Error GoTo errShowUser

    Const FUNCTION_NAME = "showUser"
   
    'rc
    showUser = False
    
    executeStatement "SELECT * FROM USERS WHERE OWNER ='" + m_oComFunc.convertStringForCharCol(sOwner, True) + "' AND USERNAME ='" + m_oComFunc.convertStringForCharCol(sUser, True) + "'"
                    

    showUser = True

    Exit Function

errShowUser:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
Public Function executeStatement(ByVal sStatement As String)
    On Error GoTo errExecuteStatement
    
    Const FUNCTION_NAME = "executeStatement"
        
    '=varibles===================================================================
    Dim results                 As New Collection
    Dim oStatements             As New BreakStatement
    '=============================================================================
    
    'rc
    executeStatement = False
    
    m_oGlobalInfo.Hourglass.State = qpHourglassOn

    Set oStatements.connection = m_oAdminCon
    
      'consider user setting for clipped result view
    If m_oGlobalInfo.userProfile.clippedResultView Then
    
        oStatements.resultMode = sapdbVirtualMode
    Else
        oStatements.resultMode = sapdbSimpleMode
    End If
    oStatements.statement = sStatement
    
    oStatements.asyncExecute = m_oGlobalInfo.userProfile.asyncExecute
    oStatements.asyncExecuteWaitTime = m_oGlobalInfo.userProfile.asyncExecuteWaitTime
    
    If oStatements.executeStatements(results, Internal) = False Then

        m_oGlobalInfo.Hourglass.State = qpHourglassOff
        Exit Function
    End If

    m_oGlobalInfo.Hourglass.State = qpHourglassOff

    If Not results Is Nothing Then

        If results.count > 0 Then
           showResults results, sStatement, sapdbVirtualMode, "", ""
        Else
            MsgBox m_oAdminCon.Error.Text + " " + LoadResString(117)
        End If
    End If
    
    'rc
    executeStatement = True
    
    Exit Function
    
errExecuteStatement:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
Public Function removeCloseButton(oForm As Form)
    On Error GoTo errRemoveCloseButton
    
    Const FUNCTION_NAME = "removeCloseButton"
    
    '=varibles===================================================================
    Dim hSysMenu As Long
    Dim nCnt     As Long
    '=============================================================================
    
    ' Get handle to our form's system menu (Restore, Maximize, Move, close etc.)
    hSysMenu = GetSystemMenu(oForm.HWnd, False)

    If hSysMenu Then
        ' Get system menu's menu count
        nCnt = GetMenuItemCount(hSysMenu)

        If nCnt Then
            ' Menu count is based on 0 (0, 1, 2, 3...)
            RemoveMenu hSysMenu, nCnt - 2, MF_BYPOSITION Or MF_REMOVE
            RemoveMenu hSysMenu, nCnt - 3, MF_BYPOSITION Or MF_REMOVE ' Remove the seperator

            DrawMenuBar oForm.HWnd  ' Force caption bar's refresh. Disabling X button
        End If
    End If
    
    Exit Function
    
errRemoveCloseButton:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getNoTimeOut
'    description:   reads the value of timeout option from the registry to the timeout checkbox in the main frame window
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Function getNoTimeOut()
    On Error GoTo errGetNoTimeOut
    
    Const FUNCTION_NAME = "getNoTimeOut"
    
    '=varibles===================================================================
    Dim bReturn      As Boolean
    Dim lKey         As Long
    Dim lDisPos      As Long
    Dim sValue       As String
    Dim sKey         As String
    '============================================================================
    
    sValue = Space$(64)
    sKey = m_oComFunc.getRegBasicKey + NOTIMEOUT + "\" + NOVERSIONDEP
        
    m_oRegFunc.showErrors = False
    bReturn = m_oRegFunc.openKey(HKEY_CURRENT_USER, sKey, KEY_ALL_ACCESS, lKey)
    
    If bReturn = True Then
    
        bReturn = m_oRegFunc.QueryValueString(lKey, NOTIMEOUT, sValue, Len(sValue))
        If bReturn Then
            bReturn = m_oComFunc.getBoolValueFromToken(Trim(sValue))
            If bReturn Then frmMain.chkNoTimeOut.Value = vbChecked
        End If
    End If
    
    m_oRegFunc.closeKey sKey, lKey
    
    Exit Function
    
errGetNoTimeOut:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      setNoTimeOut
'    description:   writes the value of the timeout checkbox in the main frame window to the registry
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Function setNoTimeOut()
    On Error GoTo errSetNoTimeOut
    
    Const FUNCTION_NAME = "setNoTimeOut"
    
    '=varibles===================================================================
    Dim lNoTimeOut   As Long
    Dim bReturn      As Boolean
    Dim lKey         As Long
    Dim lDisPos      As Long
    Dim sKey         As String
    Dim oKey         As SQLStudioUtil.Key
    Dim oVal         As SQLStudioUtil.Value
    '=============================================================================
        
    If frmMain.chkNoTimeOut.Value = vbChecked Then
    
        lNoTimeOut = 1
    Else
        lNoTimeOut = 0
    End If
    
    sKey = m_oComFunc.getRegBasicKey + NOTIMEOUT + "\" + NOVERSIONDEP
        
    m_oRegFunc.showErrors = False
    bReturn = m_oRegFunc.openKey(HKEY_CURRENT_USER, sKey, KEY_ALL_ACCESS, lKey)
        
    If bReturn = False Then _
       bReturn = m_oRegFunc.createKey(HKEY_CURRENT_USER, sKey, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, lKey, lDisPos)
    
    If bReturn = True Then _
       bReturn = m_oRegFunc.setValueString(lKey, NOTIMEOUT, Trim(Str(lNoTimeOut)))
    
    m_oRegFunc.closeKey sKey, lKey

    Exit Function
    
errSetNoTimeOut:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      timeDiff
'    description:   calculating the difference between two time values in different possible formats
'    arguments:     [in] sTimeFormat, string representig the time format to work with
'                   [in] sStart, start time
'                   [in] sEnd, end time
'    return value:  time difference as string
'
'=========================================================================================
Public Function timeDiff(sTimeFormat As String, sStart As String, sEnd As String) As String
    On Error GoTo errTimeDiff
    
    Const FUNCTION_NAME = "timeDiff"

    '=varibles===================================================================
    Dim iMilDiff   As Integer
    Dim iSecDiff   As Integer
    Dim iMinDiff   As Integer
    Dim iHourDiff  As Integer
    Dim iSecOffset As Integer
    Dim iMinOffset As Integer
    Dim sUnit      As String
    '=============================================================================
    
    iHourDiff = Mid(sEnd, 1, 2) - Mid(sStart, 1, 2)
    iMinDiff = Mid(sEnd, 4, 2) - Mid(sStart, 4, 2)
    iSecDiff = Mid(sEnd, 7, 2) - Mid(sStart, 7, 2)
    iMilDiff = Mid(sEnd, 10, 3) - Mid(sStart, 10, 3)
        
    If sTimeFormat = "m" Then
        timeDiff = Format$(iMinDiff + 60 * iHourDiff) + " min"
        Exit Function
    End If

    If sTimeFormat = "s" Then
        timeDiff = Format$(iSecDiff + 60 * iMinDiff + 3600 * iHourDiff) + " sec"
        Exit Function
    End If

    If sTimeFormat = "hh:mm:ss.xxx" Then
        If iMilDiff < 0 Then
            iSecDiff = iSecDiff - 1
            iMilDiff = iMilDiff + 1000
        End If
    
        If iSecDiff < 0 Then
            iMinDiff = iMinDiff - 1
            iSecDiff = iSecDiff + 60
        End If
    
        If iMinDiff < 0 Then
            iHourDiff = iHourDiff - 1
            iMinDiff = iMinDiff + 60
        End If
        
        If iHourDiff > 0 Then
            timeDiff = Format$(iHourDiff, "00") + ":" + _
                       Format$(iMinDiff, "00") + ":" + _
                       Format$(iSecDiff, "00") + "." + _
                       Format$(iMilDiff, "000") + " h"
            Exit Function
        End If
            
        If iMinDiff > 0 Then
            timeDiff = Format$(iMinDiff, "00") + ":" + _
                       Format$(iSecDiff, "00") + "." + _
                       Format$(iMilDiff, "000") + " min"
            Exit Function
        End If
                   
        timeDiff = Format$(iSecDiff, "00") + "." + _
                   Format$(iMilDiff, "000") + " sec"
        
        Exit Function
    End If

Exit Function

errTimeDiff:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      cmdLineOptsInit
'    description:   initis the global command line object
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Function cmdLineOptsInit()
    On Error GoTo errCmdLineOptsInit
    
    Const FUNCTION_NAME = "cmdLineOptsInit"
    
    '=varibles===================================================================
    Dim lPrefix As CmdLinePrefixConstants
    '=============================================================================
    
    'init
    lPrefix = cmlPrefixMinus
    
    m_oCML.add lPrefix, CML_OPT_HELP, True, True, "", LoadResString(401)
    m_oCML.add lPrefix, CML_OPT_NODE, True, True, LoadResString(406), LoadResString(402)
    m_oCML.add lPrefix, CML_OPT_DB, True, True, LoadResString(407), LoadResString(403)
    m_oCML.add lPrefix, CML_OPT_USER, True, True, LoadResString(408), LoadResString(404)
    m_oCML.add lPrefix, CML_OPT_VER, True, True, "", LoadResString(405)
    m_oCML.add lPrefix, CML_OPT_CM, True, True, LoadResString(409), LoadResString(410)
    m_oCML.add lPrefix, CML_OPT_TRACE, True, True, LoadResString(411), LoadResString(412)

    Exit Function
    
errCmdLineOptsInit:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function

'=========================================================================================
'    function:      showCriticalConMsg
'    description:   shows the error message
'    arguments:     oConnection: connection object
'
'    return value:  /
'
'=========================================================================================
Public Function showCriticalConMsg(oConnection As connection)
    On Error GoTo errShowCriticalConMsg
    
    Const FUNCTION_NAME = "showCriticalConMsg"
    
    '"Connection failed ..."
    setState LoadResString(100), True
    
    If Len(oConnection.Error.Text) > 0 Then
        
        oConnection.Error.showError
    Else
        If Len(oConnection.Error.sqlState) > 0 Then
        
            MsgBox oConnection.Error.sqlState
       Else
            showError142 oConnection
        End If
    End If

    Exit Function
    
errShowCriticalConMsg:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      showSettings
'    description:   shows dialog for user settings
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Function showSettings()
    On Error GoTo errShowSettings
    
    Const FUNCTION_NAME = "showSettings"
        
    '=varibles===================================================================
    Dim frmUP     As New frmUserProperties
    '============================================================================
        
    'set a connection
    Set frmUP.connection = m_oAdminCon
    'set the user profile object
    Set frmUP.userProfile = m_oGlobalInfo.userProfile
    
    'load the user profile from the database
    frmUP.loadUserProfile
    
        
    'if system tables are loaded and user is not exclusive set sqlmode, isolevel from user profile object
    If (m_oGlobalInfo.conMan.isNotExclusive And _
        m_oGlobalInfo.systemtablesLoaded) _
        Or _
        m_oGlobalInfo.conMan.isConnected = False _
    Then
        frmUP.sqlMode = m_oGlobalInfo.userProfile.appManagment.sqlMode
        frmUP.isoLevel = m_oGlobalInfo.userProfile.appManagment.isoLevel
    Else
        'set driver sqlmode, isolevel
        If (m_oGlobalInfo.conMan.driverSQLMode > 0) Then
            frmUP.sqlMode = m_oGlobalInfo.conMan.driverSQLMode
        Else
            frmUP.sqlMode = Internal
        End If
        If (m_oGlobalInfo.conMan.driverSQLMode > 0) Then
            frmUP.isoLevel = m_oGlobalInfo.conMan.driverIsoLevel
        Else
            frmUP.isoLevel = Committed
        End If
    End If
    
    'if system tables are loaded and user is not exclusive enable some properties
    'if user is disconnected enable properties
    frmUP.propertiesEnabled = ((m_oGlobalInfo.conMan.isNotExclusive And _
                               m_oGlobalInfo.systemtablesLoaded) Or _
                               m_oGlobalInfo.conMan.isConnected = False)
    
    'show dialog
    frmUP.Show vbModal, frmMain
    
    If frmUP.ok And m_oGlobalInfo.conMan.isConnected Then
       
       'enable all features
       frmMain.ctlCM.tinyMode = False
        
       'save node info of catalogtree
       m_oGlobalInfo.userProfile.expandedNodes = frmMain.ctlCM.getNodeInfo
    
       'load all catalog objects
       frmMain.ctlCM.loadCatalog _
                            m_oGlobalInfo.conMan.UID, _
                            frmMain.ctlCM.tableFilter, _
                            itemAll
                            
       frmMain.ctlCM.setNodeInfo m_oGlobalInfo.userProfile.expandedNodes, True
       
       'set the expanded nodes
       frmMain.ctlCM.setNodeInfo m_oGlobalInfo.userProfile.expandedNodes, m_oGlobalInfo.userProfile.restoreCatalogState
       
       If m_oGlobalInfo.conMan.isNotExclusive And m_oGlobalInfo.systemtablesLoaded Then
    
            'set changed values
            m_oGlobalInfo.userProfile.appManagment.sqlMode = frmUP.sqlMode
            m_oGlobalInfo.userProfile.appManagment.isoLevel = frmUP.isoLevel
       End If
    End If
        
    Unload frmUP
    Set frmUP = Nothing
    
    Exit Function
    
errShowSettings:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      changeBackSlashToSlash
'    description:   changes all backslashes in a string to slashes
'    arguments:     [in] sInput, string to be processed
'                   [out] sOutPut, new string
'    return value:  /
'
'=========================================================================================
Public Function changeBackSlashToSlash(ByVal sInput As String, sOutPut As String)
    On Error GoTo errChangeBackSlashToSlash

    Const FUNCTION_NAME = "changeBackSlashToSlash"
    
    'init output string
    sOutPut = ""

    Dim lPos As Long
        
    Do
        lPos = InStr(sInput, "\")
        If lPos > 0 Then

            sOutPut = sOutPut + Left(sInput, lPos - 1)
            sOutPut = sOutPut + "/"

            sInput = Right(sInput, Len(sInput) - lPos)
        End If
    Loop While lPos > 0 And Len(sInput) > 0

    sOutPut = sOutPut + sInput

    Exit Function

errChangeBackSlashToSlash:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      showError142
'    description:   shows the error with the resource id 142
'    arguments:     oConnection: connection object
'
'    return value:  /
'
'=========================================================================================
Function showError142(oConnection As connection)
    On Error GoTo errShowError142
    
    Const FUNCTION_NAME = "showError142"

    '"-23062 Couldn't open connection !"
    MsgBox LoadResString(142) + _
            " (" + _
            oConnection.sqlModeAsString + _
            ", " + _
            oConnection.isoLevelAsString + ")", _
            , vbCritical
    
    Exit Function
    
errShowError142:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      checkLocalRoot
'    description:   check for a valid local root
'    arguments:     [in/out] sLocalRoot to check
'
'
'    return value:  /
'
'=========================================================================================
Public Function checkLocalRoot(ByRef sLocalRoot As String)
    On Error GoTo errCheckLocalRoot

    Const FUNCTION_NAME = "checkLocalRoot"

    '=local varibles==========================
    Dim sTemp  As String
    '==========================================
        
    'if there is no local root in the the user profile, set system temp path
    If Len(sLocalRoot) < 1 Then
    
        sTemp = App.Path + "/" + App.EXEName
        changeBackSlashToSlash sTemp, sLocalRoot
    Else
        changeBackSlashToSlash sLocalRoot, sLocalRoot
        If Not m_oGlobalInfo Is Nothing Then
            'remove name of database which was added to local root in previous verions
            If m_oGlobalInfo.conMan.isConnected Then
                If Right(sLocalRoot, _
                         1 + Len(m_oGlobalInfo.conMan.database)) = "/" + m_oGlobalInfo.conMan.database Then
                    sLocalRoot = Left(sLocalRoot, Len(sLocalRoot) - Len(m_oGlobalInfo.conMan.database))
                End If
            End If
        End If
    End If
    
    Exit Function
    
errCheckLocalRoot:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      checkResult
'    description:   check if the is a valid result for the given SQL statement
'    arguments:     [in] statement - SQL statement to check
'    return value:  Boolean
'
'=========================================================================================
Public Function checkResult(statement As String) As Boolean
    On Error GoTo errCheckResult

    Const FUNCTION_NAME = "checkResult"
    
    Dim oResult   As New SQLStudioDBLayer.Result
    
    checkResult = False
    
    Set oResult.connection = m_oAdminCon
    If Not oResult.openResult(statement, Internal) Then
    
        oResult.closeResult
        Exit Function
    End If
    If Not oResult.fetch() Then
    
        oResult.closeResult
        Exit Function
    End If
    If oResult.noRows() Then
    
        oResult.closeResult
        Exit Function
    End If
    
    oResult.closeResult
    checkResult = True
    
    Exit Function
    
errCheckResult:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      procIDFromWnd
'    description:   check if the is a valid result for the given SQL statement
'    arguments:     [in] statement - SQL statement to check
'    return value:  proc id
'
'=========================================================================================
Function procIDFromWnd(ByVal HWnd As Long) As Long
    On Error GoTo errProcIDFromWnd

    Const FUNCTION_NAME = "procIDFromWnd"
    
    '=local variables=========
    Dim idProc As Long
    '=========================
   
    If HWnd = 0 Then Exit Function
   
    'get PID for this HWnd
    GetWindowThreadProcessId HWnd, idProc
   
    'Return PID
    procIDFromWnd = idProc
   
    Exit Function
    
errProcIDFromWnd:
    internalError Err, MODULE_NAME, FUNCTION_NAME
    End Function
'=========================================================================================
'    function:      setTaskIDToMainWnd
'    description:   set database task ID in frame window
'    arguments:     [in] statement - SQL statement to check
'    return value:  proc id
'
'=========================================================================================
Private Function setTaskIDToMainWnd()
    On Error GoTo errSetTaskIDToMainWnd

    Const FUNCTION_NAME = "setTaskIDToMainWnd"
    
    '=local variables=========
    Dim idProc As Long
    '=========================
    
    If frmMain.HWnd = 0 Then Exit Function
    
    idProc = procIDFromWnd(frmMain.HWnd)
    
    getDBTaskID idProc
    
    
    Exit Function
    
errSetTaskIDToMainWnd:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getDBTaskID
'    description:   check if the the necessary systemtables are installed
'    arguments:     [in] sOwner, owner of systemtable
'                   [in] sName, name  of systemtable
'
'    return value:  true if successfully
'
'=========================================================================================
Private Function getDBTaskID(lPID As Long) As Boolean
    On Error GoTo errGetDBTaskID
    
    Const FUNCTION_NAME = "getDBTaskID"
    
    '=local varibles==============================
    Dim sTask      As String
    Dim sStatement As String
    Dim oResult    As New SQLStudioDBLayer.Result
    '=============================================
    
    'rc
    getDBTaskID = False
    
    sStatement = "select PROCESS from SYSDBA.TRANSACTIONS where APPLPROCESS = " + Trim(Str(lPID))
                
    Set oResult.connection = m_oAdminCon
    oResult.limitForLong = -1
    If Not oResult.openResult(sStatement, Internal) Then Exit Function
    If Not oResult.fetch Then Exit Function
    Do While (Not oResult.noRows)
    
        oResult.GetData
        sTask = sTask + "T" + oResult.Columns(1).colValue + ","
        If Not oResult.fetch Then Exit Do
    Loop
        
    If Len(sTask) > 2 Then sTask = Left(sTask, Len(sTask) - 1)
    
    Set oResult = Nothing
    
    frmMain.Caption = frmMain.Caption + " - " + LoadResString(625) + " " + sTask
    
    getDBTaskID = True
    
    Exit Function

errGetDBTaskID:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      openDBProcInfo
'    description:   show some info of a db procedure
'    arguments:     [in] sOwner, owner of db proc
'                   [in] sItem, name of db proc
'
'    return value:  /
'
'=========================================================================================
Public Function openDBProcInfo(ByVal sOwner As String, _
                               ByVal sItem As String)
    On Error GoTo errOpenDBProcInfo
    
    Const FUNCTION_NAME = "openDBProcInfo"
    
    '=local varibles==========================
    Dim oConnection     As SQLStudioDBLayer.connection
    Dim bConnectSuccess As Boolean
    Dim sql             As String
    Dim frmInfo         As New frmDBProcInfo
    '=========================================
    
    m_oGlobalInfo.Hourglass.State = qpHourglassOn
    
    bConnectSuccess = m_oGlobalInfo.conMan.getConnection( _
                          m_oGlobalInfo.conMan.driverIsoLevel, _
                          m_oGlobalInfo.conMan.driverSQLMode, _
                          AutoCommitOn, _
                          oConnection)
    
    If Not bConnectSuccess Then
    
        m_oGlobalInfo.Hourglass.State = qpHourglassOff
        setState LoadResString(100), True
        Exit Function
    End If
    
    'check if required data is available
    sql = "SELECT DEFINITION,PACKAGE FROM DOMAIN.DBPROCEDURES WHERE OWNER = '" + _
                  m_oComFunc.convertStringForCharCol(sOwner, True) + "' AND DBPROCNAME ='" + _
                  m_oComFunc.convertStringForCharCol(sItem, True) + "' "
    If checkResult(sql) = False Then
        MsgBox LoadResString(234), vbCritical, LoadResString(230)
        Exit Function
    End If
    
    
    Load frmInfo
    frmInfo.procOwner = sOwner
    frmInfo.procName = sItem
    Set frmInfo.connection = oConnection
    Set frmInfo.globalInfo = m_oGlobalInfo
    Set frmInfo.statementList = m_oStatementList
    
    frmInfo.Caption = LoadResString(232) + " - " + sOwner + "." + sItem
    
    m_oGlobalInfo.Hourglass.State = qpHourglassOff
    
    If frmInfo.init() = False Then
    
        
        Unload frmInfo
        Set frmInfo = Nothing
        Exit Function
    End If
    
    frmInfo.Visible = True
        
    Exit Function

errOpenDBProcInfo:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      openTriggerDefinition
'    description:   show some info of a db procedure
'    arguments:     [in] sOwner, owner of the trigger
'                   [in] sItem, name of trigger
'                   [in] sTable, table of trigger
'
'    return value:  /
'
'=========================================================================================
Public Function openTriggerDefinition(ByVal sOwner As String, _
                                      ByVal sItem As String, _
                                      ByVal sTable As String)
    On Error GoTo errOpenTriggerDefinition
    
    Const FUNCTION_NAME = "openTriggerDefinition"
    
'=varibles===================================================================
    Dim bConnectSuccess      As Boolean
    Dim bLoadDefinition      As Boolean
    Dim sql                  As String
    Dim sDefinition          As String
    Dim oConnection          As SQLStudioDBLayer.connection
    Dim NewForm              As New frmDirectSQL
    Dim oResult              As New SQLStudioDBLayer.Result
    '=========================================================================
    
    If Forms.count > m_csMaxForm Then
    
        MsgBox LoadResString(115), vbCritical
        Exit Function
    End If
    
        
    m_oGlobalInfo.Hourglass.State = qpHourglassOn
    bConnectSuccess = m_oGlobalInfo.conMan.getConnection( _
                                        m_oGlobalInfo.conMan.driverIsoLevel, _
                                        m_oGlobalInfo.conMan.driverSQLMode, _
                                        AutoCommitOn, _
                                        oConnection)
                                             
    m_oGlobalInfo.Hourglass.State = qpHourglassOff
    
    If Not bConnectSuccess Then
    
        showCriticalConMsg oConnection
        Exit Function
    End If
    
    
    
    sql = "SELECT definition FROM domain.triggers WHERE triggername = '" + _
          m_oComFunc.convertStringForCharCol(sItem, True) + "' AND owner = '" + _
          m_oComFunc.convertStringForCharCol(sOwner, True) + "'" + " AND tablename = '" + _
          m_oComFunc.convertStringForCharCol(sTable, True) + "'"
          
    bLoadDefinition = True
    Set oResult.connection = m_oAdminCon
    If Not oResult.openResult(sql, Internal) Then bLoadDefinition = False
    If Not oResult.fetch() Then bLoadDefinition = False
    If oResult.noRows() Then bLoadDefinition = False
    If oResult.GetData() = False Then bLoadDefinition = False
    
    If bLoadDefinition Then
        sDefinition = oResult.Columns(1).colValue
    End If
    oResult.closeResult
        
    If bLoadDefinition = False Then
        MsgBox LoadResString(233), vbCritical, LoadResString(230)
        Exit Function
    End If
      
    'Load form
    Load NewForm
    
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.statementList = m_oStatementList
    Set NewForm.connection = oConnection
    Set NewForm.StoredFileContent = Nothing
    NewForm.Text = sDefinition
    NewForm.Caption = sItem + " " + LoadResString(111) + " " + sTable
    
    'Show form
    NewForm.Show
    NewForm.refreshFont
        
    Exit Function

errOpenTriggerDefinition:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function

'=========================================================================================
'    function:      removeFailedConnectAttempts
'    description:   read the value if if failed connects should be removed from the registry
'    arguments:     /
'
'    return value:  see description
'
'=========================================================================================
Public Function removeFailedConnectAttempts() As Boolean
    On Error GoTo errRemoveFailedConnectAttempts
    
    Const FUNCTION_NAME = "removeFailedConnectAttempts"
    
    '=varibles===================================================================
    Dim bReturn      As Boolean
    Dim lKey         As Long
    Dim lDisPos      As Long
    Dim sValue       As String
    Dim sKey         As String
    '============================================================================
    
    'rc
    removeFailedConnectAttempts = False
    
    sValue = Space$(64)
    sKey = m_oComFunc.getRegBasicKey + REMOVEFAILEDCON + "\" + NOVERSIONDEP
        
    m_oRegFunc.showErrors = False
    bReturn = m_oRegFunc.openKey(HKEY_CURRENT_USER, sKey, KEY_ALL_ACCESS, lKey)
    
    If bReturn = True Then
    
        bReturn = m_oRegFunc.QueryValueString(lKey, REMOVEFAILEDCON, sValue, Len(sValue))
        If bReturn Then
            bReturn = m_oComFunc.getBoolValueFromToken(Trim(sValue))
            If bReturn Then removeFailedConnectAttempts = True
        End If
    Else
        '!!!!! if there is no key, the value for removing the LRU Connection is true!!!!!!!!
        removeFailedConnectAttempts = True
    End If
    
    m_oRegFunc.closeKey sKey, lKey
    
    Exit Function
    
errRemoveFailedConnectAttempts:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      createGlobalInfo
'    description:   create the globalinfo object
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Function createGlobalInfo()
    On Error GoTo errCreateGlobalInfo
    
    Const FUNCTION_NAME = "createGlobalInfo"
    
    'free SQL Studio environment
    Set m_oGlobalInfo = Nothing
    
    ' Allocate new global variable to communicate
    If m_oGlobalInfo Is Nothing Then
        Set m_oGlobalInfo = New SQLStudioMan.globalInfo
    End If
    
    'check
    If m_oGlobalInfo Is Nothing Then
    
        MsgBox LoadResString(350), vbCritical
        'End
    End If
        
    Exit Function
    
errCreateGlobalInfo:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function

'=========================================================================================
'    function:      openNewInstance
'    description:   open an new sql studio instance
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Function openNewInstance()
    On Error GoTo errOpenNewInstance
    
    Const FUNCTION_NAME = "openNewInstance"

    Dim sPath As String
      
    sPath = App.Path + "\" + App.EXEName + ".EXE"
    m_oComFunc.setWindowSettings frmMain, WINDOWPLACEMENT + "\" + NOVERSIONDEP, 200, 200, 0, 0
    Shell sPath, vbNormalNoFocus
    
    Exit Function
    
errOpenNewInstance:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getVisible
'    description:   reads the Visible type window
'    arguments:     [in] sSubkey for registry
'
'    return value:  /
'
'=========================================================================================
Public Function getVisible(sSubKey As String) As Boolean
    On Error GoTo errgetVisible
    
    Const FUNCTION_NAME = "getVisible"
    
    '=varibles===================================================================
    Dim bReturn      As Boolean
    Dim lKey         As Long
    Dim lValue       As Long
    Dim sKey         As String
    '=============================================================================
    
    sKey = m_oComFunc.getRegBasicKey + sSubKey
        
    m_oRegFunc.showErrors = False
    bReturn = m_oRegFunc.openKey(HKEY_CURRENT_USER, sKey, KEY_ALL_ACCESS, lKey)
    
    If bReturn = True Then
    
        bReturn = m_oRegFunc.queryValueLong(lKey, VISIBLESUBKEY, lValue)
        If bReturn Then
            If lValue = 1 Then
                getVisible = True
            Else
                getVisible = False
            End If
        End If
        m_oRegFunc.closeKey sKey, lKey
    Else
        getVisible = True
    End If
    
    Exit Function
    
errgetVisible:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      setVisible
'    description:   writes the window settings like top,left ... from the registry
'    arguments:     [in] sSubkey for registr
'                   [in] is Visible ?
'    return value:  /
'
'=========================================================================================
Public Function setVisible(sSubKey As String, bVisible As Boolean)
    On Error GoTo errsetVisible
    
    Const FUNCTION_NAME = "setVisible"
    
    '=varibles===================================================================
    Dim bReturn      As Boolean
    Dim lValue       As Long
    Dim lKey         As Long
    Dim lDisPos      As Long
    Dim sKey         As String
    Dim oKey         As SQLStudioUtil.Key
    Dim oVal         As SQLStudioUtil.Value
    '=============================================================================
    
    sKey = m_oComFunc.getRegBasicKey + sSubKey
        
    m_oRegFunc.showErrors = False
    bReturn = m_oRegFunc.openKey(HKEY_CURRENT_USER, sKey, KEY_ALL_ACCESS, lKey)
        
    If bReturn = False Then _
       bReturn = m_oRegFunc.createKey(HKEY_CURRENT_USER, sKey, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, lKey, lDisPos)
    
    If bReturn = True Then
        If bVisible Then
            lValue = 1
        Else
            lValue = 0
        End If
        bReturn = m_oRegFunc.setValueLong(lKey, VISIBLESUBKEY, lValue)
    End If
    
    m_oRegFunc.closeKey sKey, lKey

    Exit Function
    
errsetVisible:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
Public Function setViewMenusForAllForms()
    On Error GoTo errSetViewMenusForAllForms
    
    Const FUNCTION_NAME = "setViewMenusForAllForms"
    
    Dim i As Integer
    
    For i = 0 To Forms.count - 1
    
        Forms(i).mnuViewStatusWindow.Checked = frmMain.mnuViewStatusWindow.Checked
        Forms(i).mnuViewCatalogManager.Checked = frmMain.mnuViewCatalogManager.Checked
    Next
    
    Exit Function
    
errSetViewMenusForAllForms:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      setRegistryLongValue
'    description:   write a long value to the registry
'    arguments:     [in] sSubkey for registry
'                   [in] lValue
'
'    return value:  /
'
'=========================================================================================
Public Function setRegistryLongValue(sSubKey As String, sKeyName As String, lValue As Long)
    On Error GoTo errSetRegistryLongValue
    
    Const FUNCTION_NAME = "setRegistryLongValue"
    
    '=varibles===================================================================
    Dim bReturn      As Boolean
    Dim lKey         As Long
    Dim lDisPos      As Long
    Dim sKey         As String
    Dim oKey         As SQLStudioUtil.Key
    Dim oVal         As SQLStudioUtil.Value
    '=============================================================================
    
    sKey = m_oComFunc.getRegBasicKey + sSubKey
        
    m_oRegFunc.showErrors = False
    bReturn = m_oRegFunc.openKey(HKEY_CURRENT_USER, sKey, KEY_ALL_ACCESS, lKey)
        
    If bReturn = False Then _
       bReturn = m_oRegFunc.createKey(HKEY_CURRENT_USER, sKey, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, lKey, lDisPos)
    
    If bReturn = True Then
        bReturn = m_oRegFunc.setValueLong(lKey, sKeyName, lValue)
    End If
    
    m_oRegFunc.closeKey sKey, lKey

    Exit Function
    
errSetRegistryLongValue:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function


'=========================================================================================
'    function:      setRegistryLongValue
'    description:   write a long value to the registry
'    arguments:     [in] sSubkey for registry
'                   [in] lValue
'
'    return value:  /
'
'=========================================================================================
Public Function getRegistryLongValue(sSubKey As String, sKeyName As String) As Long
    On Error GoTo errGetRegistryLongValue
    
    Const FUNCTION_NAME = "getRegistryLongValue"
    
    '=varibles===================================================================
    Dim bReturn      As Boolean
    Dim lValue       As Long
    Dim lKey         As Long
    Dim lDisPos      As Long
    Dim sKey         As String
    Dim oKey         As SQLStudioUtil.Key
    Dim oVal         As SQLStudioUtil.Value
    '=============================================================================
    
    sKey = m_oComFunc.getRegBasicKey + sSubKey
        
    m_oRegFunc.showErrors = False
    bReturn = m_oRegFunc.openKey(HKEY_CURRENT_USER, sKey, KEY_ALL_ACCESS, lKey)
    
    If bReturn = True Then
    
        bReturn = m_oRegFunc.queryValueLong(lKey, sKeyName, lValue)
        If Not bReturn Then lValue = -1
        m_oRegFunc.closeKey sKey, lKey
    End If
    
    getRegistryLongValue = lValue
    
    Exit Function
    
errGetRegistryLongValue:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function




